import pychemia
import datetime


class CodeSPRKKR:

    def __init__(self, structure):
        self.structure = structure

    def write_sys(self):
        name = self.structure.formula
        wf = open(name + '.sys', 'w')
        wf.write('system data-file created by pychemia on %s\n' % str(datetime.datetime.now()))
        wf.write(name + '.sys\n')
        wf.write('xband-version\n')
        wf.write('6.3\n')
        wf.write('dimension\n')
        wf.write('3D\n')
        sym = pychemia.crystal.CrystalSymmetry(self.structure)
        wf.write("Bravais lattice \n")
        wf.write('12 %s primitive %s %s\n' % (sym.crystal_system().lower(), sym.symbol(),
                                              sym.get_symmetry_dataset()['pointgroup']))
        wf.write('space group number (ITXC and AP)\n')
        wf.write(' %4d %4d\n' % (0, 0))
        wf.write('structure type\n')
        wf.write('UNKNOWN\n')
        wf.write('lattice parameter A  [a.u.] \n')
        a, b, c = self.structure.lattice.lengths
        au = pychemia.utils.constants.angstrom_bohr
        wf.write(' %17.12f\n' % (a * au))
        wf.write('ratio of lattice parameters  b/a  c/a\n')
        wf.write(" %17.12f %17.12f\n" % (b/a, c/a))
        wf.write("lattice parameters  a b c  [a.u.] \n")
        wf.write(" %17.12f %17.12f %17.12f\n" % (a*au, b*au, c*au))
        alpha, beta, gamma = self.structure.lattice.angles
        wf.write("lattice angles  alpha beta gamma  [deg] \n")
        wf.write(' %17.12f %17.12f %17.12f\n' % (alpha, beta, gamma))
        wf.write("primitive vectors     (cart. coord.) [A]\n")
        wf.write(" %17.12f %17.12f %17.12f\n" % tuple(self.structure.lattice.cell[0]/a))
        wf.write(" %17.12f %17.12f %17.12f\n" % tuple(self.structure.lattice.cell[1]/a))
        wf.write(" %17.12f %17.12f %17.12f\n" % tuple(self.structure.lattice.cell[2]/a))
        wf.write("number of sites NQ\n")
        wf.write(" %2d\n" % self.structure.natom)
        wf.write(" IQ ICL     basis vectors     (cart. coord.) [A] %s RWS [a.u.]  NLQ  NOQ ITOQ\n" % (20*' '))

        dm = self.structure.distance_matrix().flatten()
        dmin = min(dm[dm > 0.0]) * pychemia.utils.constants.angstrom_bohr
        rws = dmin / 2.0
        reduced = self.structure.reduced
        for i in range(self.structure.natom):
            wf.write(" %2d %3d %17.12f %17.12f %17.12f %17.12f %2d %2d %d\n" % (i+1, i+1,
                                                                                reduced[i, 0],
                                                                                reduced[i, 1],
                                                                                reduced[i, 2],
                                                                                rws, 3, 1, i+1))

        wf.write("number of sites classes NCL\n")
        wf.write(" %2d\n" % self.structure.natom)
        wf.write("ICL WYCK NQCL IQECL (equivalent sites)\n")
        for i in range(self.structure.natom):
            wf.write(" %2d   -   %2d %2d\n" % (i+1, 1, i+1))
        wf.write("number of atom types NT\n")
        wf.write(" %2d\n" % self.structure.natom)
        wf.write(" IT  ZT  TXTT  NAT  CONC  IQAT (sites occupied)\n")
        for i in range(self.structure.natom):
            atomic_z = pychemia.utils.periodic.atomic_number(self.structure.symbols[i])
            wf.write(" %2d %3d %6s %7d %5.3f %2d\n" % (i+1, atomic_z, self.structure.symbols[i], 1, 1.0, i+1))
        wf.close()

    def write_pot(self):
        strdate = str(datetime.datetime.now())
        natom = self.structure.natom
        name = self.structure.formula
        ret = """*******************************************************************************
HEADER    'SCF-start data created by pychemia %s'
*******************************************************************************
TITLE     'SPR-KKR calculation for %s'
SYSTEM    %s
PACKAGE   SPRKKR
FORMAT    6  (21.05.2007)
*******************************************************************************
GLOBAL SYSTEM PARAMETER
NQ                 %d
NT                 %d
NM                 %d
IREL               3
*******************************************************************************
SCF-INFO
INFO      NONE
SCFSTATUS START
FULLPOT   F
BREITINT  F
NONMAG    F
ORBPOL    NONE
EXTFIELD  F
BLCOUPL   F
BEXT          0.0000000000
SEMICORE  F
LLOYD     F
NE                32
IBZINT             2
NKTAB              0
XC-POT    VWN
SCF-ALG   BROYDEN2
SCF-ITER           0
SCF-MIX       0.2000000000
SCF-TOL       0.0000100000
RMSAVV    999999.0000000000
RMSAVB    999999.0000000000
EF            0.0000000000
VMTZ          0.0000000000
""" % (strdate, name, name, natom, natom, natom)

        sym = pychemia.crystal.CrystalSymmetry(self.structure)
        a, b, c = self.structure.lattice.lengths
        au = pychemia.utils.constants.angstrom_bohr

        ret += """*******************************************************************************
LATTICE
SYSDIM       3D
SYSTYPE      BULK
BRAVAIS           12        %s       primitive      %s    %s
""" % (sym.crystal_system().lower(), sym.symbol(), sym.get_symmetry_dataset()['pointgroup'])

        ret += "ALAT          %15.10f\n" % (a*au)

        ret += "A(1)          %15.10f %15.10f %15.10f\n" % tuple(self.structure.lattice.cell[0] / a)
        ret += "A(2)          %15.10f %15.10f %15.10f\n" % tuple(self.structure.lattice.cell[1] / a)
        ret += "A(3)          %15.10f %15.10f %15.10f\n" % tuple(self.structure.lattice.cell[2] / a)

        ret += """*******************************************************************************
SITES
CARTESIAN T
BASSCALE      1.0000000000    1.0000000000    1.0000000000
        IQ      QX              QY              QZ
"""
        reduced = self.structure.reduced
        for i in range(natom):
            ret += "%d  %15.10f %15.10f %15.10f\n" % (i+1, reduced[i, 0], reduced[i, 1], reduced[i, 2])

        ret += """*******************************************************************************
OCCUPATION
        IQ     IREFQ       IMQ       NOQ  ITOQ  CONC
"""

        for i in range(natom):
            ret += "%10d %10d %10d %10d %5d %5f\n" % (i+1, i+1, i+1, 1, i+1, 1.0)

        ret += """*******************************************************************************
REFERENCE SYSTEM
NREF              %d
      IREF      VREF            RMTREF
""" % natom

        for i in range(natom):
            ret += " %10d %15.10f %15.10f\n" % (i+1, 4.0, 0.0)

        ret += """*******************************************************************************
MAGNETISATION DIRECTION
KMROT              0
QMVEC         0.0000000000    0.0000000000    0.0000000000
        IQ      QMTET           QMPHI
"""
        for i in range(natom):
            ret += " %10d %15.10f %15.10f\n" % (i+1, 0.0, 0.0)

        ret += """*******************************************************************************
MESH INFORMATION
MESH-TYPE EXPONENTIAL
   IM      R(1)            DX         JRMT      RMT        JRWS      RWS
        """

        for i in range(natom):
            ret += " %5d %15.10f %15.10f %d %15.10f %d %15.10f\n" % (i+1, 0.0000010000, 0.0205501520,    0,
                                                                     2.2661445689, 721, 2.6660524340)

        ret += """*******************************************************************************
TYPES
   IT     TXTT        ZT     NCORT     NVALT    NSEMCORSHLT
"""
        for i in range(natom):
            atomic_z = pychemia.utils.periodic.atomic_number(self.structure.symbols[i])
            ret += " %2d %6s %5d %7d %5d %2d\n" % (i+1, self.structure.symbols[i], atomic_z, 18, 8-i, 0)

        wf = open(name + '.pot', 'w')
        wf.write(ret)
        wf.close()



from abc import ABCMeta, abstractmethod
import os
import subprocess


class Codes:
    __metaclass__ = ABCMeta

    def __init__(self):
        self.stdin_file = None
        self.stdout_file = None
        self.stderr_file = None
        self.stdin_filename = None
        self.stdout_filename = None
        self.stderr_filename = None
        self.binary = None
        self.runner = None
        self.workdir = None

    @abstractmethod
    def initialize(self, structure, workdir=None, kpoints=None, binary=None):
        pass

    @abstractmethod
    def set_inputs(self):
        pass

    @abstractmethod
    def get_outputs(self):
        pass

    @abstractmethod
    def finalize(self):
        pass

    def run(self, use_mpi=False, omp_max_threads=0, mpi_num_procs=1):
        """
        Execute the binary and return a reference to the subprocess
        created

        :param use_mpi: If mpirun will be called to execute the binary
        :param omp_max_threads: Number of OpenMP threads to be created
                                by default the environment variable
                                OMP_NUM_THREADS is not changed
        :param mpi_num_procs: Number of MPI processes
        :return:
        """
        cwd = os.getcwd()
        os.chdir(self.workdir)
        if omp_max_threads > 0:
            os.environ["OMP_NUM_THREADS"] = str(omp_max_threads)
        if self.stdin_filename is not None:
            self.stdin_file = open(self.stdin_filename, 'r')
        if self.stdout_filename is not None:
            self.stdout_file = open(self.stdout_filename, 'w')
        if self.stderr_filename is not None:
            self.stderr_file = open(self.stderr_filename, 'w')
        if use_mpi:
            sp = subprocess.Popen(['mpirun', '-n', str(mpi_num_procs), self.binary],
                                  stdout=self.stdout_file, stderr=self.stderr_file, stdin=self.stdin_file)
        else:
            sp = subprocess.Popen(self.binary, stdout=self.stdout_file, stderr=self.stderr_file, stdin=self.stdin_file)
        os.chdir(cwd)
        self.runner = sp
        return sp



from __future__ import unicode_literals
import unittest
# import unittest2 as unittest
import os
from pychemia import HAS_PYMONGO

if HAS_PYMONGO:
    from pychemia.population import LJCluster, RelaxStructures, OrbitalDFTU, NonCollinearMagMoms, \
        RealFunction


def has_connection():
    if not HAS_PYMONGO:
        return False
    import pymongo
    try:
        maxSevSelDelay = 1
        client = pymongo.MongoClient("localhost", serverSelectionTimeoutMS=maxSevSelDelay)
        client.server_info()  # force connection on a request as the
        # connect=True parameter of MongoClient seems
        # to be useless here
        return True
    except pymongo.errors.ServerSelectionTimeoutError as err:
        # do whatever you need
        print(err)
        return False


def funx2(x):
    return x ** 2


class PopulationTest(unittest.TestCase):
    def test_ljcluster(self):
        """
        Tests (pychemia.population.LJCluster)                        :
        """
        if not has_connection():
            return
        popu = LJCluster('test', 'Ne4')
        popu.add_random()
        popu.add_random()
        popu.pcdb.clean()

    def notest_structure(self):
        """
        Tests (pychemia.population.RelaxStructures)                  :
        """
        if not has_connection():
            return
        popu = RelaxStructures('test', 'NaCl')
        popu.add_random()
        popu.add_random()
        popu.pcdb.clean()

    def test_noncoll(self):
        """
        Tests (pychemia.population.NonCollinearMagMoms)              :
        """
        if not has_connection():
            return
        popu = NonCollinearMagMoms('test', source_dir='pychemia/test/data/vasp_02')
        popu.add_random()
        popu.add_random()
        popu.pcdb.clean()

    def notest_dftu(self):
        """
        Tests (pychemia.population.OrbitalDFTU)                      :
        """
        if not has_connection():
            return
        print(os.getcwd())
        popu = OrbitalDFTU('test', './data/abinit_05/abinit.in')
        popu.add_random()
        popu.add_random()
        popu.pcdb.clean()

    def test_euclidean(self):
        """
        Tests (pychemia.population.RealFunction)                     :
        """
        if not has_connection():
            return
        popu = RealFunction(funx2, 2, [-1, 1])
        popu.add_random()
        popu.add_random()


if __name__ == "__main__":
    unittest.main()



import os
import sys
from pychemia import HAS_PYMATGEN
from .structure import Structure
from pychemia.code.vasp import read_poscar
from pychemia.code.abinit import InputVariables


def structure_from_file(structure_file):
    st = None
    if not os.path.isfile(structure_file):
        print("ERROR: Could not open file '%s'" % structure_file)
        sys.exit(1)
    if structure_file[-4:].lower() == 'json':
        st = Structure.load_json(structure_file)
    elif structure_file[-3:].lower() == 'cif' and HAS_PYMATGEN:
        import pychemia.external.pymatgen
        st = pychemia.external.pymatgen.cif2structure(structure_file)[0]
    elif structure_file[-6:].lower() == 'poscar':
        st = read_poscar(structure_file)
    elif structure_file[-6:].lower() == 'contcar':
        st = read_poscar(structure_file)
    elif structure_file[-6:].lower() == 'abinit.in':
        av = InputVariables(structure_file)
        av.get_structure()
    else:
        try:
            st = read_poscar(structure_file)
        except ValueError:
            print("ERROR: Could not extract structure from file '%s'" % structure_file)
            exit(1)
    return st



import os as _os
import subprocess as _subprocess
from .utils import netcdf2dict, psp_name


class AbiFiles:
    """
    Read an 'abinit.files' file and extract the
    names of input, output filenames and put them
    into dictionary called 'files'.
    """
    basedir = "."
    filename = "abinit.files"
    files = {'in': 'abinit.in',
             'out': 'abinit.out',
             'tmpin': 'abinit-i',
             'tmpout': 'abinit-o',
             'tmp': 'abinit',
             'psps': ''}

    def __init__(self, *args, **kwargs):
        """
        Creates an abifiles object

        Args:
            args:
                If args[0] is an existing filename, assume that as the
                name of the 'abinit.files' and their path is the
                basedir, otherwise will consider that as the basedir.
            kwargs:
                Valid keywords are: basedir,filename,files,in.out,
                tmpin,tmpout,tmp and psps
        """
        self.inp = None
        if len(args) == 1:
            if _os.path.isfile(args[0]):
                (self.basedir, self.filename) = _os.path.split(args[0])
                if self.basedir == "":
                    self.basedir = "."
                inputfile = open(args[0], "r")
                self.files['in'] = inputfile.readline()[:-1]
                self.files['out'] = inputfile.readline()[:-1]
                self.files['tmpin'] = inputfile.readline()[:-1]
                self.files['tmpout'] = inputfile.readline()[:-1]
                self.files['tmp'] = inputfile.readline()[:-1]
                self.files['psps'] = map(str.strip, inputfile.readlines())

            elif _os.path.isdir(args[0]):
                self.basedir = args[0]

            elif not _os.path.exists(args[0]):
                self.basedir = args[0]

        if len(kwargs) > 0:
            if 'basedir' in kwargs:
                self.basedir = kwargs['basedir']
                print(self.filename)
            if 'filename' in kwargs:
                self.filename = kwargs['filename']
            if 'files' in kwargs:
                self.files['in'] = kwargs['files'] + ".in"
                self.files['out'] = kwargs['files'] + ".out"
                self.files['tmpin'] = kwargs['files'] + "-i"
                self.files['tmpout'] = kwargs['files'] + "-o"
                self.files['tmp'] = kwargs['files']
            for x in ['in', 'out', 'tmpin', 'tmpout', 'tmp', 'psps']:
                if x in kwargs:
                    self.files[x] = kwargs[x]

    def write(self, filename):
        """
        Write the file 'filename' with the format of an
        usual '.files'

        :param filename: (str) Filename to write the 'abinit.files' file
        """
        wf = open(filename, 'w')
        wf.write(self.__str__())
        wf.close()

    def create(self):
        """
        Create the '.files' file and the directory
        if it does not exists
        """

        if not _os.path.exists(self.basedir):
            _os.makedirs(self.basedir)
        self.write(self.basedir + "/" + self.filename)
        # Write the input file
        if self.inp is not None:
            self.inp.write(self.get_input_filename())

    def __str__(self):
        """
        String version of the object, shows the same
        info as the contents of the .files file
        """
        ret = ''
        for x in ['in', 'out', 'tmpin', 'tmpout', 'tmp']:
            ret = ret + self.files[x] + "\n"
        for x in self.files['psps']:
            ret = ret + x + "\n"
        return ret

    def __repr__(self):
        """
        Representation of an abifiles object
        """
        ret = "basedir:           " + self.basedir + '\n' + \
              "filename:          " + self.filename + '\n' + \
              "        \-> in     " + self.files['in'] + '\n' + \
              "        \-> out    " + self.files['out'] + '\n' + \
              "        \-> tmpin  " + self.files['tmpin'] + '\n' + \
              "        \-> tmpout " + self.files['tmpout'] + '\n' + \
              "        \-> tmp    " + self.files['tmp'] + '\n'
        for x in self.files['psps']:
            ret = ret + "        \-> psps   " + x + '\n'
        return ret

    def get_input_filename(self):
        """
        Return the input filename
        """
        return self.basedir + "/" + self.files['in']

    def get_output_filename(self):
        """
        Return the output filename
        """
        return self.basedir + "/" + self.files['out']

    def get_out_filename(self):
        """
        Return the OUT.nc filename
        """
        return self.basedir + "/" + self.files['tmpout'] + '_OUT.nc'

    def clean(self):
        """
        Remove all the output
        """
        if _os.path.isdir(self.basedir):
            _os.remove(self.get_out_filename())
            outfile = self.files['out']
            outs = [x for x in _os.listdir(self.basedir) if x[:len(outfile)] == outfile]
            for i in outs:
                _os.remove(self.basedir + '/' + i)

    def cleanall(self):
        if _os.path.isdir(self.basedir):
            outfile = self.files['out']
            outs = [x for x in _os.listdir(self.basedir)]
            for i in outs:
                _os.remove(self.basedir + '/' + i)
            self.create()

    def set_input(self, inp):
        """
        Associate an inputvars object to the abifiles object
        """
        self.inp = inp

    def get_output(self):
        """
        Return the output as a dictionary
        """
        return netcdf2dict(self.get_out_filename())

    def execute(self, abinit_binary):
        """
        Utility that copy a given script and execute the given
        command inside the directory
        """
        cwd = _os.getcwd()
        _os.chdir(self.basedir)
        abifile = open(self.filename)
        logfile = open('abinit.log', 'w')
        _subprocess.call([abinit_binary], stdin=abifile, stdout=logfile)
        logfile.close()
        abifile.close()
        _os.chdir(cwd)

    def set_psps(self, exchange='LDA', kind='FHI'):
        """
        Set the pseudopotentials acording to
        given exchange and kind
        The pair (exchange,kind) could be:
        ('LDA','FHI')
        ('LDA','TM')
        ('GGA','FHI')
        """
        if self.inp is None:
            print('ABINIT input file not declared, the pseudopotentials cannot be set')
        else:
            self.files['psps'] = []
            pspdir = _os.getenv('HOME') + '/.abinit/' + exchange + '_' + kind
            if isinstance(self.inp.variables['znucl'], (int, float)):
                lstznucl = [self.inp.variables['znucl']]
            else:
                lstznucl = self.inp.variables['znucl']

            for i in lstznucl:
                self.files['psps'].append(pspdir + '/' + psp_name(i, exchange, kind))



#!/usr/bin/env python

# This script download several sets of pseudopotentials
# form the Website of ABINIT
# The pseudopotentials will be located inside the directory
# '.abinit' of the user home directory, just aside of the
# tarballs.
# AE are the Hirshfeld All Electron (AE) densities

from __future__ import print_function
import os
import ftplib

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen
import tarfile
import time

from pychemia import HAS_SCIPY

if HAS_SCIPY:
    from pychemia.code.abinit import psp_name
else:
    raise ImportError('scipy could not be found')


def get_rpath_psp(kind, exchange, atomicnumber=None):
    """
    Get the ftp path to get the PSP

    :param kind: (str) Source of Pseudopotentials
    :param exchange: (str) 'LDA' or 'GGA'
    :param atomicnumber: (int) Atomic Number
    :return:
    """
    rpath = None
    if kind == 'FHI' and exchange == 'LDA':
        rpath = '/pub/abinitio/Psps/LDA_FHI/'
    elif kind == 'FHI' and exchange == 'GGA':
        rpath = '/pub/abinitio/Psps/GGA_FHI/'
    elif kind == 'GTH' and exchange == 'LDA':
        rpath = '/pub/abinitio/Psps/LDA_GTH/'
    elif kind == 'CORE' and exchange == 'LDA':
        rpath = '/pub/abinitio/Psps/LDA_Core/'
    elif kind == 'HGH' and exchange == 'LDA':
        rpath = '/pub/abinitio/Psps/LDA_HGH/'
    elif kind == 'HGH' and exchange == 'GGA':
        rpath = '/pub/abinitio/Psps/GGA_HGH/'
    elif kind == 'TM' and exchange == 'LDA':
        if atomicnumber is None:
            raise ValueError('Atomic number required')
        rpath = '/pub/abinitio/Psps/LDA_TM.psps/' + str(atomicnumber).zfill(2) + '/'
    elif kind == 'AE' and exchange == 'DEN':
        rpath = '/pub/abinitio/Psps/AE_DEN/'
    elif kind == 'FC' and exchange == 'DEN':
        rpath = '/pub/abinitio/Psps/FC_DEN/'
    elif kind == 'PAW' and exchange == 'LDA':
        rpath = 'http://www.abinit.org/downloads/PAW2/ATOMICDATA/JTH-LDA-atomicdata.tar.gz'
    elif kind == 'PAW' and exchange == 'GGA':
        rpath = 'http://www.abinit.org/downloads/PAW2/ATOMICDATA/JTH-PBE-atomicdata.tar.gz'
    else:
        print('Not know kind of PSP')
    return rpath


def get_all_psps(basedir, exchange, kind):
    directory = basedir + os.sep + exchange + '_' + kind
    if not os.path.isdir(directory):
        os.mkdir(directory)
    if kind == 'PAW':
        rpath = get_rpath_psp(kind, exchange)
        filename = rpath.split('/')[-1]
        if not os.path.isfile(directory + '/' + filename):
            u = urlopen(rpath)
            f = open(directory + os.sep + filename, 'wb')
            meta = u.info()
            file_size = int(meta.get("Content-Length")[0])
            print("Downloading: %s Bytes: %s" % (filename, file_size))
            file_size_dl = 0
            block_sz = 8192
            while True:
                readed_buffer = u.read(block_sz)
                if not readed_buffer:
                    break
                file_size_dl += len(readed_buffer)
                f.write(readed_buffer)
                status = r"%10d  [%3.2f%%]" % (file_size_dl, file_size_dl * 100. / file_size)
                status += chr(8) * (len(status) + 1)
                print(status, end='')
            f.close()
            print('\n')
        try:
            tar = tarfile.open(directory + '/' + filename, 'r:gz')
            for item in tar:
                if not os.path.exists(item.name):
                    tar.extract(item, path=directory)
        except tarfile.ReadError:
            name = os.path.basename(filename)
            print(name[:name.rfind('.')], '<filename>')

    elif kind == 'HGH':
        while True:
            succeed = True
            ftp = ftplib.FTP('ftp.abinit.org')  # connect to host, default port
            ftp.login()  # user anonymous, passwd anonymous@
            ftp.cwd('pub/abinitio/Psps/LDA_HGH/')
            for filename in ftp.nlst():
                if not os.path.exists(directory + '/' + filename) or os.path.getsize(directory + '/' + filename) == 0:
                    print('Getting %s' % filename)
                    try:
                        ftp.retrbinary('RETR ' + filename, open(directory + '/' + filename, 'wb').write)
                    except ftplib.error_perm:
                        print('Failed to get ', filename)
                        succeed = False
            ftp.close()
            if succeed:
                break
    else:
        ftp = ftplib.FTP('ftp.abinit.org')  # connect to host, default port
        ftp.login()  # user anonymous, passwd anonymous@
        missing_psps = []
        for i in range(1, 113):
            if kind == 'GTH' and i > 17:
                continue
            if kind == 'CORE' and i not in [6, 7]:
                continue
            if kind == 'FHI' and exchange == 'LDA' and i in [57, 59, 63, 65, 66, 67, 90, 94, 102, 110, 111, 112]:
                continue
            if kind == 'TM' and exchange == 'LDA' and i in [104, 105, 106, 107, 108, 109, 110, 111, 112]:
                continue
            if kind == 'FHI' and exchange == 'GGA' and i in [57, 59, 63, 65, 66, 67, 90, 94, 102, 110, 111, 112]:
                continue
            if kind == 'AE' and exchange == 'DEN' and i in [57, 59, 63, 65, 66, 67, 90, 94, 102, 110, 111, 112]:
                continue
            if kind == 'GTH' and exchange == 'LDA' and i in [2, 10]:
                continue
            if kind == 'FC' and exchange == 'DEN' and i in [63, 65, 66, 67, 110, 111, 112]:
                continue
            filename = psp_name(i, exchange, kind)
            if not os.path.isfile(directory + '/' + filename) or os.path.getsize(directory + '/' + filename) == 0:
                print('Getting...' + filename)
                nofile = True
                while nofile:
                    retr = 'RETR ' + get_rpath_psp(kind, exchange, i) + filename
                    try:
                        ftp.retrbinary(retr, open(directory + '/' + filename, 'wb').write)
                        nofile = False
                        if os.path.getsize(directory + '/' + filename) == 0:
                            os.remove(directory + '/' + filename)
                    except ftplib.error_perm:
                        print('Could not download ' + retr)
                        missing_psps.append(i)
                        ftp.close()
                        time.sleep(5)
                        print('Reconnecting...')
                        if os.path.isfile(directory + '/' + filename):
                            os.remove(directory + '/' + filename)
                        ftp = ftplib.FTP('ftp.abinit.org')  # connect to host, default port
                        ftp.login()  # user anonymous, passwd anonymous@
                        nofile = False
        ftp.close()
        if len(missing_psps) > 0:
            print("kind == '%s' and exchange == '%s' and i in %s" % (kind, exchange, missing_psps))


if __name__ == '__main__':

    home = os.environ['HOME']
    basedir = home + "/.abinit"

    if not os.path.isdir(basedir):
        os.mkdir(basedir)

    for exchange in ['LDA', 'GGA', 'DEN']:
        print('=> ' + exchange)
        lista = []
        if exchange == 'LDA':
            lista = ['FHI', 'TM', 'GTH', 'PAW', 'CORE', 'HGH']
        elif exchange == 'GGA':
            lista = ['FHI', 'HGH', 'PAW']
        elif exchange == 'DEN':
            lista = ['AE', 'FC']

        for kind in lista:
            print('--> ' + kind)
            get_all_psps(basedir, exchange, kind)



import sys

data = open(sys.argv[1]).readlines()

Sections = {}

for i in range(len(data)):
    Variable = ''
    Section = ''
    if data[i][:8] == 'Variable':
        Variable = data[i].split()[1]
        for j in range(1, 5):
            if data[i + j][:8] == 'Section ':
                Section = data[i + j][8:].split('::')[0].strip()
                break
        if Section == '':
            print('Error finding section for ', Variable)

        if Section not in Sections.keys():
            print('New Section ', Section)
            Sections[Section] = [Variable]
        else:
            Sections[Section].append(Variable)

variables = open('octopus_variables.conf', 'w')
for i in sorted(Sections.keys()):
    variables.write('\n[' + i + ']\n')
    for j in sorted(Sections[i]):
        variables.write(j + '\n')
variables.close()



import os
import time
import numpy as np
from threading import Thread


class ObjectiveFunction:
    def __init__(self):
        self.population = None

    def initialize(self, population):
        self.population = population

    def ids_sorted(self, selection):
        values = np.array([self.population.value(i) for i in selection])
        argsort = np.argsort(values)
        return np.array(selection)[argsort]

    def get_values(self, selection):
        ret = {}
        for i in selection:
            ret[i] = self.population.value(i)
        return ret


class Evaluator:
    def __init__(self):
        self.process = None
        self.thread = None
        self.population = None

    @property
    def is_running(self):
        if self.thread is not None:
            return self.thread.is_alive()
        else:
            return False

    def initialize(self, population):
        self.population = population

    def evaluate(self, i):
        x = self.population.coordinate(i)
        y = self.population.function(x)
        if y is not None:
            self.population.set_value(i, y)
            self.population.evaluated.append(i)

    def run(self):

        def worker(evaluator, population):
            while True:
                for i in population.actives:
                    if not population.is_evaluated(i):
                        evaluator.evaluate(i)
                time.sleep(1)
                if os.path.exists('stop'):
                    os.remove('stop')
                    return

        # self.process = Process(target=worker, args=(self.population,))
        # self.process.start()
        self.thread = Thread(target=worker, args=(self, self.population,))
        self.thread.daemon = True
        self.thread.start()

    def stop(self):
        if self.process is not None and not self.process.is_alive():
            self.process.terminate()



from math import sqrt, ceil
import numpy as np
from pychemia.utils.serializer import PyChemiaJsonable, generic_serializer

"""
Definition of the set of k-points in reciprocal space
"""

__author__ = "Guillermo Avendano-Franco"
__copyright__ = "Copyright 2014"
__version__ = "1.1"
__maintainer__ = "Guillermo Avendano-Franco"
__email__ = "gtux.gaf@gmail.com"
__status__ = "Development"
__date__ = "November 5, 2014"


class KPoints(PyChemiaJsonable):
    """
    Defines an object that contains information about kpoints mesh
    There are two mutually exclusive options:

    Explicit array of kpoints with associated weights OR
    a specification of a grid with an optional set of shifts.

    The default is one single kpoint in gamma
    """

    def __init__(self, kmode='gamma',
                 kpoints_list=None, weights=None,
                 shifts=None, grid=None,
                 kvertices=None, intermediates=None):
        """
        The KPoints class is designed to  store the description of k-points in a code agnostic way.

        There are 5 possible kmodes to describe a set of k-points, currently those modes are not interchangable,
        but that could change in the future.

        The kmodes are 'cartesian', 'reduced', 'gamma', 'monkhorst-pack' and 'path'.

        Kmode 'cartesian' and 'reduced' uses explicit declaration of a grid of k-points.
        In the case of 'cartesian' the points are vectors in k-space.
        In the case of 'reduced' the points are the factors for the vectors defining the reciprocal lattice.
        The variable 'kpoints_list' is a numpy array with a shape of ('nkpt', 3)
        The variable 'weigths' is a numpy array with the weights associated to each point.
        by default all the weights are equal to  1.

        Kmode 'gamma' and 'monkhorst-pack' uses an uniform grid to describe the k-points mesh.
        The variable 'grid' describes the number of points on each dimension an it is a numpy array
        with 3 values.
        The variable 'shifts' describe the shift done to the grid according to the mode selected.
        The internal value 'nkpt' will be simply the product of the three values of the grid.
        Typically ab-initio packages will try to exclude from that grid equivalent k-points due to
        symmetry considerations.

        Kmode 'path' is specially useful for band-structure calculations, in this case the variable
        'kvertices' contains all the vertices, usually high symmetry points and the variable
        'intermediates' contains the number of k-points created between two consecutive vertices.

        When a Kpoints object is created without any arguments, the default will be a kmode='gamma',
        with a grid=[1, 1, 1] and shifts=[0, 0, 0]

        :param kmode: The options are 'cartesian', 'reduced', 'gamma', 'monkhorst-pack'
        :param kpoints_list: Explicit list of k-points (Only for 'cartesian' and 'reduced')
        :param weights: List of weights associated to each k-point (Only for 'cartesian' and 'reduced')
        :param grid: Number of kpoints on each direction (Only for 'gamma' and 'monkhorst-pack')
        :param shifts: Shift applied to the grid (Only for 'gamma' and 'monkhorst-pack')
        :param kvertices: List of vertices for a path of k-points (Only for 'path')
        :param intermediates: Number of intermediates between two vertices (Only for 'path')

        :rtype : Kpoints object
        """
        if kmode.lower() not in ['cartesian', 'reduced', 'gamma', 'monkhorst-pack', 'path']:
            raise ValueError("kmode must be 'cartesian', 'reduced', 'gamma', 'monkhorst-pack' or 'path'")

        self.kmode = kmode.lower()

        # All the variables initially None
        self.kpoints_list = None
        self.weights = None
        self.grid = None
        self.shifts = None
        self.kvertices = None
        self.intermediates = None

        # Explicit list of k-points
        if self.kmode in ['cartesian', 'reduced']:
            self.set_kpoints_list(kpoints_list, weights)
        # Grid specification
        elif self.kmode in ['gamma', 'monkhorst-pack']:
            self.set_grid(grid, shifts)
        # Path along the K-space
        elif self.kmode == 'path':
            if kvertices is None:
                self.kvertices = np.array([[0, 0, 0]])
            else:
                if np.array(kvertices).shape == (3,):
                    self.kvertices = np.array(kvertices).reshape((-1, 3))
                elif np.array(kvertices).shape[1] == 3:
                    self.kvertices = np.array(kvertices)
                else:
                    raise ValueError("Wrong value for kvertices, should be an array with shape (nkpt,3)")
            if intermediates is None:
                self.intermediates = 1
            else:
                self.intermediates = int(intermediates)

    def add_kpt(self, kpoint, weight=1):
        """
        Add one extra kpoint to the kpoints_list in the case of
        kmode 'cartesian' or 'reduced'

        :param kpoint: (list, numpy.array) The kpoint to be added
        :param weight: (int, float) the weight associated to that point
        """
        kpoint = list(kpoint)
        assert len(kpoint) == 3
        assert (self.kmode in ['cartesian', 'reduced'])
        if kpoint not in self.kpoints_list:
            self.kpoints_list.append(kpoint)
            self.weights = np.append(self.weights, weight)
        else:
            position = self.kpoints_list.index(kpoint)
            self.weights[position] = weight

    def set_kpoints_list(self, kpoints_list, weights=None):
        """
        Set an explicit list of kpoints with a proper check of correct arguments

        :param kpoints_list: Explicit list of k-points (Only for 'cartesian' and 'reduced')
        :param weights: List of weights associated to each k-point (Only for 'cartesian' and 'reduced')
        """
        assert (self.kmode in ['cartesian', 'reduced'])
        if kpoints_list is None:
            self.kpoints_list = [[0, 0, 0]]
        else:
            if np.array(kpoints_list).shape == (3,):
                self.kpoints_list = np.array(kpoints_list).reshape((-1, 3))
            elif np.array(kpoints_list).shape[1] == 3:
                self.kpoints_list = np.array(kpoints_list)
            else:
                raise ValueError("Wrong value for kpoints_list, should be an array with shape (nkpt,3)")
        self.kpoints_list = generic_serializer(self.kpoints_list)
        nkpt = len(self.kpoints_list)
        if weights is None:
            self.weights = np.ones(nkpt)
        elif len(np.array(weights)) == nkpt:
            self.weights = np.array(weights)
        else:
            raise ValueError("Wrong value for weights, should be an array with shape (nkpt,)")
        self.weights = list(self.weights)

    def set_grid(self, grid, shifts=None):
        """
        Set a grid of kpoints with a proper check of correct arguments

        :param grid: Number of kpoints on each direction (Only for 'gamma' and 'monkhorst-pack')
        :param shifts: Shift applied to the grid (Only for 'gamma' and 'monkhorst-pack')
        """

        assert (self.kmode in ['gamma', 'monkhorst-pack'])
        if grid is None:
            self.grid = [1, 1, 1]
        elif len(grid) == 3:
            self.grid = [int(x) for x in grid]
        else:
            raise ValueError("Wrong value for grid, should be an array with shape (-1,3)")
        if shifts is None:
            self.shifts = [0, 0, 0]
        else:
            self.shifts = generic_serializer(np.array(shifts).reshape(3))

    @property
    def number_of_kpoints(self):
        return self.nkpt

    @property
    def nkpt(self):
        """
        Number of kpoints
        For a grid the value is the product of the number of kpoints
        on each direction.
        """
        if self.kmode in ['cartesian', 'reduced']:
            return len(self.kpoints_list)
        elif self.kmode in ['gamma', 'monkhorst-pack']:
            return np.prod(self.grid)
        elif self.kmode == 'path':
            return len(self.kvertices) * self.intermediates

    def __str__(self):
        """
        String representation of the KPoints object
        """
        kp = ' Mode  : ' + self.kmode + '\n'
        if self.kmode in ['cartesian', 'reduced']:
            kp += ' Number of k-points: ' + str(self.nkpt) + '\n\n'
            for i in range(self.nkpt):
                kp += (" %10.5f %10.5f %10.5f %15.5f\n"
                       % (self.kpoints_list[i][0], self.kpoints_list[i][1], self.kpoints_list[i][2], self.weights[i]))
        elif self.kmode in ['gamma', 'monkhorst-pack']:
            kp += ' Grid  : ' + str(self.grid) + '\n'
            kp += ' Shift : ' + str(self.shifts) + '\n'
        elif self.kmode == 'path':
            kp += ' Number of intermediates: ' + str(self.intermediates) + '\n\n'
            for vertex in self.kvertices:
                kp += " %10.5f %10.5f %10.5f\n" % (vertex[0], vertex[1], vertex[2])
        return kp

    @property
    def to_dict(self):

        ret = {}
        for i in list(self.__dict__.keys()):
            if eval('self.' + i) is not None:
                ret[i] = eval('self.' + i)
        return ret

    @classmethod
    def from_dict(cls, json_dict):
        if json_dict['kmode'] in ['cartesian', 'reduced']:
            return cls(kmode=json_dict['kmode'], kpoints_list=json_dict['kpoints_list'],
                       weights=json_dict['weights'])
        elif json_dict['kmode'] in ['gamma', 'monkhorst-pack']:
            return cls(kmode=json_dict['kmode'], grid=json_dict['grid'], shifts=json_dict['shifts'])
        elif json_dict['kmode'] == 'path':
            return cls(kmode=json_dict['kmode'], kvertices=json_dict['kvertices'],
                       intermediates=json_dict['intermediates'])

    @staticmethod
    def max_kpoints(structure, kpoints_per_atom=1000):
        """
        Returns the maximal grid of kpoints allowed during the
        kpoints convergence test. Based on the routine used in
        pymatgen.

        :param structure: (pychemia.Structure) The structure to compute the maximal grid
        :param kpoints_per_atom: (int) number of kpoints per atom
        :return:
        """

        lengths = [sqrt(sum(map(lambda y: y ** 2, structure.cell[i]))) for i in range(3)]
        ngrid = kpoints_per_atom / structure.natom
        mult = (ngrid * lengths[0] * lengths[1] * lengths[2]) ** (1.0 / 3.0)
        num_div = [int(ceil(1.0 / lengths[i] * mult)) for i in range(3)]
        num_div = [i if i > 0 else 1 for i in num_div]
        return num_div[0], num_div[1], num_div[2]

    def get_density_of_kpoints(self, lattice):

        assert (self.kmode in ['gamma', 'monkhorst-pack'])
        vol = 1.0 / lattice.volume
        return self.number_of_kpoints / vol

    @classmethod
    def optimized_grid(cls, lattice, kp_density=1E5, nkpoints=None, force_odd=False):
        """
        Returns a KPoints object with kmode='gamma' with a grid optimized for a given
         density or number of kpoints.
         If force_odd is True the grid will be such that there are odd numbers on each direction
         such that 'gamma' is always included on the grid.

        :param lattice: (pychemia.Lattice) A PyChemia Lattice object, if the argument is a pychemia.Structure
                        object, its lattice will be used instead.
        :param kp_density: (float) A density of kpoints (default: 1E5)
        :param nkpoints: (int) Aproximate number of kpoints to use, if this argument is present the kp_density
                                will be ignored
        :param force_odd: (bool) If True, force the grid to be odd for the three directions.
        """
        if nkpoints is not None:
            kp_density = None

        # Reciprocal lattice object
        rlattice = lattice.reciprocal()

        if nkpoints is not None:
            vol = 1.0 / lattice.volume
            kp_density = nkpoints / vol

        factor = int(kp_density ** 1.0 / 3.0)

        while True:
            ini_density = kp_density

            grid = np.array([int(max(ceil(factor * rlattice.a), 1)),
                             int(max(ceil(factor * rlattice.b), 1)),
                             int(max(ceil(factor * rlattice.c), 1))])

            # The volume in reciprocal space is the inverse of the volume in real space
            fin_density = float(np.prod(grid) * lattice.volume)

            if fin_density > ini_density and factor > 0:
                factor -= 1
            else:
                break

        if force_odd:
            for i in range(3):
                if grid[i] % 2 == 0:
                    grid[i] += 1
        return cls(kmode='gamma', grid=grid)



import datetime
import os
import socket
import subprocess
import xml.etree.ElementTree as ElementTree


class PBSRunner:
    def __init__(self, workdir, filename='batch.pbs'):

        self.template = None
        self.walltime = None
        self.template = None
        self.queue = None
        self.mail = None
        self.message = None
        self.walltime = None
        self.ppn = None
        self.nodes = None
        self.workdir = workdir
        self.filename = filename
        if self.workdir[-1] == os.sep:
            self.workdir = self.workdir[:-1]
        self.name = os.path.basename(self.workdir)

    def initialize(self, nodes=1, ppn=2, walltime=None, message='ae', mail=None, queue=None):

        if walltime is None:
            walltime = [12, 0, 0]
        self.set_walltime(walltime)
        self.nodes = nodes
        self.ppn = ppn
        self.message = message
        self.mail = mail
        self.queue = queue

    def set_walltime(self, walltime):

        if len(walltime) == 1:
            walltime = [0] + walltime
        if len(walltime) == 2:
            walltime = [0] + walltime
        if len(walltime) == 3:
            walltime = [0] + walltime

        self.walltime = walltime

    def set_template(self, template):
        if os.path.isfile(template):
            self.template = open(template).read()
        elif isinstance(template, str):
            self.template = template

    def write_pbs(self):

        wf = open(self.workdir + os.sep + self.filename, 'w')
        wt = self.walltime
        wf.write("""#!/bin/sh

#PBS -N %s
#PBS -l nodes=%d:ppn=%d
#PBS -l walltime=%d:%02d:%02d
#PBS -m %s
#PBS -k n
""" % (self.name, self.nodes, self.ppn, wt[0] * 24 + wt[1], wt[2], wt[3], self.message))

        if self.mail is not None:
            wf.write("#PBS -M %s\n" % self.mail)
        if self.queue is not None:
            wf.write("#PBS -q %s\n" % self.queue)
        wf.write('\ncd $PBS_O_WORKDIR\n')

        if self.template is not None:
            wf.write("%s\n" % self.template)
        wf.close()

    def submit(self, priority=0):
        cwd = os.getcwd()
        os.chdir(self.workdir)
        returncode = subprocess.call(["qsub", "%s" % self.filename, '-p', '%d' % priority])
        if returncode != 0:
            print('Some error happended:', returncode)
        os.chdir(cwd)


def get_jobs(user):
    data = subprocess.check_output(['qstat', '-x', '-f', '-u', user])
    xmldata = ElementTree.fromstring(data)
    jobs = [i.find('Job_Name').text for i in xmldata.findall('Job')]
    return jobs


def report_cover():
    ret = 'PyChemia Execution Report\n'
    ret += '=========================\n\n'
    ret += 'Hostname\n'
    ret += '--------\n\n'
    ret += socket.gethostname() + '\n\n'
    ret += 'Date\n'
    ret += '----\n'
    dt = datetime.datetime.now()
    ret += dt.strftime("%A, %d. %B %Y %I:%M%p") + '\n\n'

    ret += 'PBS VARIABLES\n'
    ret += '-------------\n\n'
    for x in ['PBS_O_HOST', 'PBS_SERVER', 'PBS_O_QUEUE', 'PBS_O_WORKDIR', 'PBS_ARRAYID', 'PBS_ENVIRONMENT',
              'PBS_JOBID', 'PBS_JOBNAME', 'PBS_NODEFILE', 'PBS_QUEUE']:
        if os.getenv(x) is not None:
            ret += x + ' = ' + os.getenv(x) + '\n'
    return ret



import os
import json
import time
import numpy as np
from pychemia import pcm_log, HAS_MATPLOTLIB
from pychemia.crystal import KPoints
from ..vasp import VaspJob
from ..outcar import read_vasp_stdout
from ..kpoints import read_kpoints
from ..poscar import read_poscar
from ...tasks import Task

__author__ = 'Guillermo Avendano-Franco'


class Convergence:
    def __init__(self, energy_tolerance):

        self.convergence_info = []
        self.energy_tolerance = energy_tolerance

    def _best_value(self, variable):
        if not self.is_converge:
            print('Convergence not completed')
            return None
        else:
            return self.convergence_info[-3][variable]

    @property
    def is_converge(self):
        if self.convergence_info is None or len(self.convergence_info) < 3:
            return False

        energies = [x['free_energy'] for x in self.convergence_info]
        if len(energies) > 2 and abs(max(energies[-3:]) - min(energies[-3:])) >= self.energy_tolerance:
            return False
        else:
            return True

    def _convergence_plot(self, variable, xlabel, title, figname, annotate):

        import matplotlib.pyplot as plt
        if not self.is_converge:
            print('Convergence not executed')
            return

        x = [idata[variable] for idata in self.convergence_info]
        y = [idata['free_energy'] for idata in self.convergence_info]
        plt.figure(figsize=(10, 8))
        plt.clf()
        plt.plot(x, y, 'rd-')
        dy = self.energy_tolerance
        sup_dy = min(y[-3:]) + dy
        low_dy = max(y[-3:]) - dy
        xlims = plt.xlim()
        plt.plot(xlims, [sup_dy, sup_dy], '0.5')
        plt.plot(xlims, [low_dy, low_dy], '0.5')
        plt.fill_between(xlims, [low_dy, low_dy], [sup_dy, sup_dy], color='0.9', alpha=0.5)
        assert (self.convergence_info is not None)
        for idata in self.convergence_info:
            plt.annotate(s=str(idata[annotate]), xy=(idata[variable], idata['free_energy']), size=10)

        plt.xlim(*xlims)
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel('Free Energy [eV]')
        plt.savefig(figname)
        return plt.gca()

    def _convergence_save(self, filename):

        if not self.is_converge:
            print('Convergence not executed')
            return

        wf = open(filename, 'w')
        json.dump(self.convergence_info, wf, sort_keys=True, indent=4, separators=(',', ': '))
        wf.close()

    def _convergence_load(self, filename):

        if not os.path.isfile(filename):
            raise ValueError('File not found: %s', filename)
        rf = open(filename, 'r')

        data = json.load(rf)
        self.output = data['output']
        self.task_params = data['task_params']
        self.convergence_info = self.output['convergence']
        self.energy_tolerance = self.task_params['energy_tolerance']
        rf.close()


class ConvergenceCutOffEnergy(Task, Convergence):
    def __init__(self, structure, workdir='.', kpoints=None, binary='vasp', energy_tolerance=1E-3,
                 increment_factor=0.2, initial_encut=1.3):

        self.structure = structure
        self.workdir = workdir
        self.binary = binary
        self.increment_factor = increment_factor
        self.initial_encut = initial_encut
        if kpoints is None:
            kp = KPoints.optimized_grid(self.structure.lattice, kp_density=1E4, force_odd=True)
            self.kpoints = kp
        else:
            self.kpoints = kpoints
        Convergence.__init__(self, energy_tolerance)
        self.task_params = {'energy_tolerance': self.energy_tolerance, 'increment_factor': self.increment_factor,
                            'initial_encut': self.initial_encut}
        Task.__init__(self, structure=structure, task_params=self.task_params, workdir=workdir, binary=binary)

    def run(self, nparal=4):

        self.started = True
        vj = VaspJob()
        vj.initialize(self.structure, self.workdir, self.kpoints, binary=self.binary)
        energies = []
        if not self.is_converge:
            x = self.initial_encut
        else:
            x = self.convergence_info[-3]['factor']
        self.convergence_info = []
        while True:
            vj.clean()
            vj.job_static()
            vj.input_variables.set_density_for_restart()
            vj.input_variables.set_encut(ENCUT=x, POTCAR=self.workdir + os.sep + 'POTCAR')
            vj.input_variables.variables['NBANDS'] = nparal * ((30 + self.structure.valence_electrons()) / nparal + 1)
            vj.input_variables.set_ismear(self.kpoints)
            vj.input_variables.variables['SIGMA'] = 0.2
            vj.input_variables.variables['ISPIN'] = 2
            vj.set_inputs()
            encut = vj.input_variables.variables['ENCUT']
            print('Testing ENCUT = %7.3f' % encut)
            vj.run(use_mpi=True, mpi_num_procs=nparal)
            pcm_log.debug('Starting VASP')
            while True:
                energy_str = ''
                filename = self.workdir + os.sep + 'vasp_stdout.log'
                if os.path.exists(filename):
                    vasp_stdout = read_vasp_stdout(filename=filename)
                    if len(vasp_stdout['data']) > 2:
                        scf_energies = [i[2] for i in vasp_stdout['data']]
                        energy_str = ' %7.3f' % scf_energies[1]
                        for i in range(1, len(scf_energies)):
                            if scf_energies[i] < scf_energies[i - 1]:
                                energy_str += ' >'
                            else:
                                energy_str += ' <'
                        pcm_log.debug(energy_str)

                if vj.runner is not None and vj.runner.poll() is not None:
                    filename = self.workdir + os.sep + 'vasp_stdout.log'
                    if os.path.exists(filename):
                        vasp_stdout = read_vasp_stdout(filename=filename)
                        if len(vasp_stdout['data']) > 2:
                            scf_energies = [i[2] for i in vasp_stdout['data']]
                            energy_str += ' %7.3f' % scf_energies[-1]
                            pcm_log.debug(energy_str)
                    pcm_log.debug('Execution complete')
                    break
                time.sleep(5)
            vj.get_outputs()
            free_energy = vj.outcar.final_data['energy']['free_energy']
            print('encut= %7.3f  free_energy: %9.6f' % (encut, free_energy))
            self.convergence_info.append({'free_energy': free_energy, 'encut': encut, 'factor': x})
            energies.append(free_energy)
            if len(energies) > 2 and abs(max(energies[-3:]) - min(energies[-3:])) < self.energy_tolerance:
                self.success = True
                break
            x = round(x + x * self.increment_factor, 2)
        self.output = {'convergence': self.convergence_info, 'best_encut': self.best_encut}
        self.finished = True

    @property
    def best_encut(self):
        return self._best_value('encut')

    def plot(self, filedir=None, file_format='pdf'):
        if filedir is None:
            filedir = self.workdir
        figname = filedir + os.sep + 'convergence.' + file_format
        return self._convergence_plot(variable='encut', xlabel='ENCUT', title='ENCUT Convergence', figname=figname,
                                      annotate='encut')

    def load(self, filename=None):
        if filename is None:
            filename = self.workdir + os.sep + 'task.json'
        self._convergence_load(filename=filename)
        self.initial_encut = self.task_params['initial_encut']
        self.increment_factor = self.task_params['increment_factor']
        self.finished = True

    def report(self, file_format='html'):

        from lxml.builder import ElementMaker, E

        if not os.path.isdir(self.report_dir):
            os.mkdir(self.report_dir)

        self.plot(filedir=self.report_dir, file_format='jpg')

        element_maker = ElementMaker(namespace=None, nsmap={None: "http://www.w3.org/1999/xhtml"})
        html = element_maker.html(E.head(E.title("VASP Energy cut-off Convergence")),
                                  E.body(E.h1("VASP Energy cut-off Convergence"),
                                         E.h2('Structure'),
                                         E.pre(str(self.structure)),
                                         E.h2('Convergence'),
                                         E.p(E.img(src='convergence.jpg', width="800", height="600", alt="Forces")),
                                         ))

        return self.report_end(html, file_format)


class ConvergenceKPointGrid(Task, Convergence):
    def __init__(self, structure, workdir='.', binary='vasp', energy_tolerance=1E-3, recover=False, encut=1.3):

        self.structure = structure
        self.workdir = workdir
        self.binary = binary
        self.initial_number = 12
        self.convergence_info = None
        self.encut = encut
        Convergence.__init__(self, energy_tolerance)
        if recover:
            self.recover()
        self.task_params = {'energy_tolerance': self.energy_tolerance, 'encut': self.encut}
        Task.__init__(self, structure=structure, task_params=self.task_params, workdir=workdir, binary=binary)

    def recover(self):
        kpoints_file = self.workdir + os.sep + 'KPOINTS'
        poscar_file = self.workdir + os.sep + 'POSCAR'
        if os.path.isfile(kpoints_file) and os.path.isfile(poscar_file):
            structure = read_poscar(poscar_file)
            kpoints = read_kpoints(kpoints_file)
            density = kpoints.get_density_of_kpoints(structure.lattice)
            self.initial_number = int(density ** (1.0 / 3.0)) - 1

    def run(self, nparal=4):

        self.started = True
        vj = VaspJob()
        kp = KPoints()
        vj.initialize(self.structure, self.workdir, kp, binary=self.binary)
        grid = None
        energies = []
        if not self.is_converge:
            n = self.initial_number
        else:
            n = self.convergence_info[-3]['kp_n']
        self.convergence_info = []
        while True:
            density = n ** 3
            kp = KPoints.optimized_grid(self.structure.lattice, kp_density=density, force_odd=True)
            pcm_log.debug('Trial density: %d  Grid: %s' % (density, kp.grid))
            if np.sum(grid) != np.sum(kp.grid):
                grid = kp.grid
                vj.set_kpoints(kp)
                vj.clean()
                vj.job_static()
                vj.input_variables.set_density_for_restart()
                vj.input_variables.set_encut(ENCUT=self.encut, POTCAR=self.workdir + os.sep + 'POTCAR')
                vj.input_variables.variables['NBANDS'] = nparal * ((30 + self.structure.valence_electrons()) /
                                                                   nparal + 1)
                vj.input_variables.set_ismear(kp)
                vj.input_variables.variables['SIGMA'] = 0.2
                vj.input_variables.variables['ISPIN'] = 2
                vj.set_inputs()
                vj.run(use_mpi=True, mpi_num_procs=nparal)
                while True:
                    energy_str = ''
                    filename = self.workdir + os.sep + 'vasp_stdout.log'
                    if os.path.exists(filename):
                        vasp_stdout = read_vasp_stdout(filename=filename)
                        if len(vasp_stdout['data']) > 2:
                            scf_energies = [i[2] for i in vasp_stdout['data']]
                            energy_str = ' %7.3f' % scf_energies[1]
                            for i in range(1, len(scf_energies)):
                                if scf_energies[i] < scf_energies[i - 1]:
                                    energy_str += ' >'
                                else:
                                    energy_str += ' <'
                            pcm_log.debug(energy_str)

                    if vj.runner is not None and vj.runner.poll() is not None:
                        filename = self.workdir + os.sep + 'vasp_stdout.log'
                        if os.path.exists(filename):
                            vasp_stdout = read_vasp_stdout(filename=filename)
                            if len(vasp_stdout['data']) > 2:
                                scf_energies = [i[2] for i in vasp_stdout['data']]
                                energy_str += ' %7.3f' % scf_energies[-1]
                                pcm_log.debug(energy_str)
                        break
                    time.sleep(5)
                vj.get_outputs()
                energy = vj.outcar.final_data['energy']['free_energy']
                energies.append(energy)
                print('kp_density= %10d kp_grid= %15s free_energy= %9.6f' % (density, grid, energy))
                self.convergence_info.append({'free_energy': vj.outcar.final_data['energy']['free_energy'],
                                              'kp_grid': list(grid),
                                              'kp_density': density,
                                              'kp_n': n})
                if len(energies) > 2 and abs(max(energies[-3:]) - min(energies[-3:])) < self.energy_tolerance:
                    self.success = True
                    break
            n += 2
        self.output = {'convergence': self.convergence_info, 'best_kp_grid': list(grid)}
        self.finished = True

    def plot(self, filedir=None, file_format='pdf'):
        if filedir is None:
            filedir = self.workdir
        figname = filedir + os.sep + 'convergence.' + file_format

        return self._convergence_plot(variable='kp_density', xlabel='K-points density', title='KPOINTS Convergence',
                                      figname=figname, annotate='kp_grid')

    def load(self, filename=None):
        if filename is None:
            filename = self.workdir + os.sep + 'task.json'
        self._convergence_load(filename=filename)
        self.encut = self.task_params['encut']
        self.finished = True

    @property
    def best_kpoints(self):

        if not self.is_converge:
            print('Convergence not completed')
            return None
        else:
            kp = KPoints.optimized_grid(self.structure.lattice, kp_density=self.convergence_info[-3]['kp_density'],
                                        force_odd=True)
            return kp

    def report(self, file_format='html'):
        from lxml.builder import ElementMaker, E

        if not os.path.isdir(self.report_dir):
            os.mkdir(self.report_dir)

        self.plot(filedir=self.report_dir, file_format='jpg')

        element_maker = ElementMaker(namespace=None, nsmap={None: "http://www.w3.org/1999/xhtml"})
        html = element_maker.html(E.head(E.title("VASP K-point grid Convergence")),
                                  E.body(E.h1("VASP K-point grid Convergence"),
                                         E.h2('Structure'),
                                         E.pre(str(self.structure)),
                                         E.h2('Convergence'),
                                         E.p(E.img(src='convergence.jpg', width="800", height="600", alt="Forces")),
                                         ))

        return self.report_end(html, file_format)



import unittest
import pychemia.utils.metaheuristics


class MetaheuristicFunctionTest(unittest.TestCase):

    def test_metaheuristic(self):
        """
        Tests (pychemia.utils.metaheuristics)                        :
        """
        for i in ['Sphere', 'Ackley', 'Rosenbrock', 'Beale', 'GoldsteinPrice', 'Booth', 'BukinN6', 'Matyas', 'LeviN13',
                  'ThreeHump', 'Easom', 'CrossInTray', 'Eggholder', 'HolderTable', 'McCormick', 'SchafferN2',
                  'SchafferN4', 'StyblinskiTang']:
            func = eval('pychemia.utils.metaheuristics.'+i+'()')

            print(i)
            print(func.mindim)
            print(func.minimum(func.mindim))
            print(func.fminimum(func.mindim))



import pychemia
import os


def notest_analysis():
    """
    Tests (pychemia.analysis.analysis)                           :
    """
    st = pychemia.utils.samples.Al2O3()
    sa = pychemia.analysis.StructureAnalysis(st, radius=20)
    struc_dist_x, fp_oganov = sa.fp_oganov(delta=0.1, sigma=0.1)
    assert len(fp_oganov) == 3


def notest_match():
    """
    Tests (pychemia.analysis.match)                              :
    """
    st = pychemia.utils.samples.Al2O3()
    st2 = st.supercell((2, 3, 4))
    sm = pychemia.analysis.StructureMatch(st, st2)
    sm.match_size()
    sm.match_shape()
    assert sm.structure1.natom == sm.structure2.natom


def test_distances():
    """
    Tests (pychemia.analysis.match)                              :
    """
    print(os.getcwd())
    st = pychemia.io.xyz.load('pychemia/test/data/xyz/chlorophyll.xyz')
    sa = pychemia.analysis.StructureAnalysis(st)
    distances = sa.all_distances()
    assert len(distances) == int(st.natom*(st.natom+1)/2)




import doctest


def test_symmetry():
    """
    DocTests (pychemia.crystal.symmetry)                         :
    """
    import pychemia
    if pychemia.HAS_SPGLIB:
        import pychemia.crystal
        dt = doctest.testmod(pychemia.crystal.symmetry, verbose=True)
        assert dt.failed == 0


def test_lattice():
    """
    DocTests (pychemia.crystal.lattice)                          :
    """
    import pychemia.crystal.lattice
    dt = doctest.testmod(pychemia.crystal.lattice, verbose=True)
    assert dt.failed == 0



#!/usr/bin/env python
"""
This example shows how to automatize the execution
of ABINIT starting with just the input file "t44.in"

The abinit.files is written with the right
locations for the pseudopotentials and the results
are post-process using the output in NetCDF format
"abinit-o_OUT.nc"
"""
from __future__ import print_function
import os
import shutil
import json
import tempfile
import subprocess
import pychemia

if pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC:
    import pychemia.code.abinit


def test_example2():
    """
    Example of a multiple calc                                   :
    """
    if not (pychemia.HAS_SCIPY and pychemia.HAS_SCIENTIFIC):
        return

    path = 'pychemia/test/data'
    assert (os.path.isdir(path))

    if which('abinit') is None:
        print('The executable "abinit" is not in the PATH')
        print('Using the results of a previous calc')
        check_results(path + os.sep + 'abinit_04')
        return

    workdir = tempfile.mkdtemp()
    print("Work directory: %s" % workdir)
    assert (os.path.isfile(path + '/abinit_04/t44.in'))
    av = pychemia.code.abinit.InputVariables(path + '/abinit_04/t44.in')
    print('Original input:\n%s' % av)
    abifiles = pychemia.code.abinit.AbiFiles(workdir)
    abifiles.set_input(av)
    abifiles.set_psps('LDA', 'FHI')
    abifiles.create()

    res = []
    wf = open(workdir + '/results.json', 'w')
    cwd = os.getcwd()
    os.chdir(workdir)

    for i in range(3):
        av.set_value('ecut', av.get_value('ecut') + 3)
        print('Computing convergence study with ecut=%f ' % av.get_value('ecut'), end='')
        if i > 0:
            av.set_value('irdwfk', 1)
        av.write(abifiles.get_input_filename())
        abifile = open(workdir + '/abinit.files')
        logfile = open(workdir + '/abinit.log', 'w')
        subprocess.call(['abinit'], stdin=abifile, stdout=logfile)
        if os.path.isfile('abinit-o_WFK'):
            shutil.copyfile('abinit-o_WFK', 'abinit-i_WFK')
        data = pychemia.code.abinit.netcdf2dict(workdir + '/abinit-o_OUT.nc')
        os.rename(workdir + '/abinit-o_OUT.nc', '%s/abinit-o_OUT.nc_%d' % (workdir, i))
        res.append({'ecut': data['ecut'], 'etotal': data['etotal']})
        print('Total energy: %f' % data['etotal'])
    os.chdir(cwd)
    json.dump(res, wf)
    wf.close()
    check_results(workdir)
    shutil.rmtree(workdir)


def check_results(workdir):
    res = json.load(open(workdir + os.sep + 'results.json'))
    assert (res[0]['etotal'] + 4.19954643154 < 1E-6)
    assert (res[1]['etotal'] + 4.19954643154 < 1E-6)
    assert (res[2]['etotal'] + 4.19954643154 < 1E-6)


def which(program):
    """
    Search for the presence of an executable
    Found in: http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
    """
    import os

    def is_exe(filep):
        return os.path.isfile(filep) and os.access(filep, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None


if __name__ == '__main__':
    test_example2()



import os
import unittest
from pychemia import pcm_log, HAS_PYMONGO
from pychemia.population import NonCollinearMagMoms
from pychemia.searcher import HarmonySearch, FireFly, GeneticAlgorithm
from pychemia.test import has_local_mongo
import logging


class SearcherTest(unittest.TestCase):

    def test_harmony(self):
        """
        Tests (pychemia.searcher.harmony) with NonCollinearMagMoms   :
        """
        logging.basicConfig(level=logging.DEBUG)
        if not HAS_PYMONGO:
            print('Could not load pymongo, leaving now')
            return
        else:
            if not has_local_mongo():
                return

        pcm_log.debug('HarmonySearch')

        source = 'pychemia/test/data/vasp_02'
        assert os.path.isfile(source + os.sep + 'INCAR')
        assert os.path.isfile(source + os.sep + 'POSCAR')
        popu = NonCollinearMagMoms('test', source, debug=True)
        popu.pcdb.clean()
        searcher = HarmonySearch(popu, generation_size=16, stabilization_limit=5)
        searcher.run()
        popu.pcdb.clean()

    def test_firefly(self):
        """
        Tests (pychemia.searcher.firefly) with NonCollinearMagMoms   :
        """
        logging.basicConfig(level=logging.DEBUG)
        if not HAS_PYMONGO:
            print('Could not load pymongo, leaving now')
            return
        else:
            if not has_local_mongo():
                return

        pcm_log.debug('HarmonySearch')

        source = 'pychemia/test/data/vasp_02'
        assert os.path.isfile(source + os.sep + 'INCAR')
        assert os.path.isfile(source + os.sep + 'POSCAR')
        popu = NonCollinearMagMoms('test', source, debug=True)
        popu.pcdb.clean()
        searcher = FireFly(popu, generation_size=16, stabilization_limit=5)
        searcher.run()
        popu.pcdb.clean()

    def test_genetic(self):
        """
        Tests (pychemia.searcher.genetic) with NonCollinearMagMoms   :
        """
        logging.basicConfig(level=logging.DEBUG)
        if not HAS_PYMONGO:
            print('Could not load pymongo, leaving now')
            return
        else:
            if not has_local_mongo():
                return

        pcm_log.debug('HarmonySearch')

        source = 'pychemia/test/data/vasp_02'
        assert os.path.isfile(source + os.sep + 'INCAR')
        assert os.path.isfile(source + os.sep + 'POSCAR')
        popu = NonCollinearMagMoms('test', source, debug=True)
        popu.pcdb.clean()
        searcher = GeneticAlgorithm(popu, generation_size=16, stabilization_limit=5)
        searcher.run()
        popu.pcdb.clean()



import os
import shutil
import pychemia
import tempfile
import unittest


class MyTestCase(unittest.TestCase):

    def test_incar(self):
        """
        Tests (pychemia.code.vasp) [INCAR parsing and writing]       :
        """
        print(os.getcwd())
        iv = pychemia.code.vasp.read_incar('pychemia/test/data/vasp_01/INCAR')
        self.assertEqual(len(iv), 12)
        self.assertEqual(iv.EDIFF, 1E-7)
        wf = tempfile.NamedTemporaryFile()
        iv.write(wf.name)
        wf.close()
        iv4dir = pychemia.code.vasp.read_incar('pychemia/test/data/vasp_01')
        self.assertEqual(iv, iv4dir)
        self.assertRaises(ValueError, pychemia.code.vasp.read_incar, 'pychemia/test/data')
        iv3 = pychemia.code.vasp.InputVariables(variables={'EDIFF': 1E-6})
        self.assertEqual(iv3['EDIFF'], 1E-6)
        iv = pychemia.code.vasp.read_incar('pychemia/test/data/vasp_02')
        iv.EDIFF *= 1.3
        td = tempfile.mkdtemp()
        pychemia.code.vasp.write_incar(iv, td)
        self.assertRaises(ValueError, iv.write_key, 'EDIF')
        shutil.rmtree(td)

    def test_bad_outcar(self):
        """
        Tests (pychemia.code.vasp) [corrupted VASP OUTCAR]           :
        """
        vo = pychemia.code.vasp.VaspOutput('pychemia/test/data/vasp_04/OUTCAR')
        self.assertTrue(vo.is_finished)

    def test_encut_setup(self):
        """
        Tests (pychemia.code.vasp) [ENCUT setup]                     :
        """
        iv = pychemia.code.vasp.read_incar('pychemia/test/data/vasp_06')
        iv.set_encut(ENCUT=1.2, POTCAR='pychemia/test/data/vasp_06/POTCAR')
        self.assertEqual(iv.ENCUT, 307)
        iv.set_rough_relaxation()
        self.assertEqual(iv.EDIFFG, -1E-2)
        iv.set_mit_settings()

    def test_vaspjob(self):
        """
        Tests (pychemia.code.vasp) [VaspJob]                         :
        """
        td = tempfile.mkdtemp()
        st = pychemia.code.vasp.read_poscar('pychemia/test/data/vasp_06')
        kp = pychemia.code.vasp.read_kpoints('pychemia/test/data/vasp_06')
        self.assertEqual(kp.number_of_kpoints, 693)
        iv = pychemia.code.vasp.read_incar('pychemia/test/data/vasp_06')
        vj = pychemia.code.vasp.VaspJob()
        vj.initialize(st, workdir=td, kpoints=kp)
        vj.set_input_variables(iv)
        vj.write_poscar()
        vj.write_kpoints()
        vj.write_incar()
        shutil.rmtree(td)

    def test_outcar(self):
        """
        Tests (pychemia.code.vasp) [outcar]                          :
        """
        vo = pychemia.code.vasp.VaspOutput('pychemia/test/data/vasp_06/OUTCAR')
        self.assertEqual(vo.get_memory_used()['grid'], (1028.0, 'kBytes'))
        self.assertAlmostEqual(vo.to_dict['energy'], -19.67192646)
        print(vo)
        self.assertTrue(vo.has_forces_stress_energy())

    def test_poscar(self):
        """
        Tests (pychemia.code.vasp) [poscar]                          :
        """
        # Temporal directory for outputs
        tmpdir = tempfile.mkdtemp()

        # Read a POSCAR by directory
        st = pychemia.code.vasp.read_poscar('pychemia/test/data/vasp_06')
        self.assertEqual(st.natom, 4)

        # Opening old format POSCAR without POTCAR
        with self.assertRaises(ValueError) as context:
            st = pychemia.code.vasp.read_poscar('pychemia/test/data/vasp_07/POSCAR')

        st = pychemia.code.vasp.read_poscar('pychemia/test/data/vasp_08/POSCAR_old')
        self.assertEqual(st.natom, 2)

        st = pychemia.code.vasp.read_poscar('pychemia/test/data/vasp_08/POSCAR_new')
        self.assertEqual(st.natom, 2)

        with self.assertRaises(ValueError) as context:
            pychemia.code.vasp.write_potcar(st, filepath=tmpdir + os.sep + 'POTCAR', basepsp='/no/existing/path')

        with self.assertRaises(ValueError) as context:
            pychemia.code.vasp.write_potcar(st, filepath=tmpdir + os.sep + 'POTCAR', basepsp='pychemia/test/data')

        cwd = os.getcwd()
        os.chdir('pychemia/test/data/vasp_07')
        st = pychemia.code.vasp.read_poscar('POSCAR_new')
        os.chdir(cwd)
        self.assertEqual(st.natom, 44)

        st = pychemia.code.vasp.read_poscar('pychemia/test/data/vasp_07/POSCAR_alt')

        pychemia.code.vasp.write_poscar(st, tmpdir + os.sep + 'POSCAR1')
        pychemia.code.vasp.write_poscar(st, tmpdir + os.sep + 'POSCAR2', direct=False)
        pychemia.code.vasp.write_poscar(st, tmpdir + os.sep + 'POSCAR3', newformat=False)

        st = pychemia.code.vasp.read_poscar(tmpdir + os.sep + 'POSCAR1')
        self.assertAlmostEqual(st.volume, 584.47161926043907)
        sym = pychemia.crystal.CrystalSymmetry(st)
        self.assertEqual(sym.symbol(), 'C2/c')
        st = pychemia.code.vasp.read_poscar(tmpdir + os.sep + 'POSCAR2')
        self.assertAlmostEqual(st.volume, 584.47161926043907)
        sym = pychemia.crystal.CrystalSymmetry(st)
        self.assertEqual(sym.symbol(), 'C2/c')
        st = pychemia.code.vasp.read_poscar(tmpdir + os.sep + 'POSCAR3')
        self.assertAlmostEqual(st.volume, 584.47161926043907)
        sym = pychemia.crystal.CrystalSymmetry(st)
        self.assertEqual(sym.symbol(), 'C2/c')
        pychemia.code.vasp.write_potcar(st, filepath=tmpdir + os.sep + 'POTCAR', basepsp='pychemia/test/data')
        pychemia.code.vasp.get_potcar_info(tmpdir + os.sep + 'POTCAR')

        shutil.rmtree(tmpdir)




#!/usr/bin/env python

import os
import sys
import pychemia


def helper():
    print(""" Plot the density of states from a given list of files
   Use:

       plot_dos.py [--figname 'DensityOfStates.pdf' ]
                   [--set_energy_range min max ]
                   [--set_figsize figwidth figheight]
                   [--help ]
                   file1.dat file2.dat ...
   """)


if __name__ == '__main__':

    # Script starts from here
    if len(sys.argv) < 2:
        helper()
        sys.exit(1)

    figname = 'DensityOfStates.pdf'
    minenergy = None
    maxenergy = None
    figheight = None
    figwidth = None
    filelist = []
    doslist = []

    for i in range(1, len(sys.argv)):
        if sys.argv[i].startswith('--'):
            option = sys.argv[i][2:]
            # fetch sys.argv[1] but without the first two characters
            if option == 'figname':
                figname = sys.argv[i + 1]
            elif option == 'help':
                helper()
                sys.exit()
            elif option == 'set_energy_range':
                minenergy = float(sys.argv[i + 1])
                maxenergy = float(sys.argv[i + 2])
            elif option == 'set_figsize':
                figwidth = float(sys.argv[i + 1])
                figheight = float(sys.argv[i + 2])
            else:
                print('Unknown option. --' + option)
        elif os.path.isfile(sys.argv[i]) and sys.argv[i][-4:] == '.dat':
            filelist.append(sys.argv[i])

    print(figname)
    for i in filelist:
        a = pychemia.visual.DensityOfStates().read(i)
        doslist.append(a)

    if len(doslist) == 0:
        helper()
        sys.exit(1)
    elif len(doslist) == 1:
        fig, ax = pychemia.visual.plot_one_dos(doslist[0], ax=None, horizontal=True, figwidth=figwidth,
                                               figheight=figheight)
        # fig.savefig(figname)
    else:
        fig, ax = pychemia.visual.plot_many_dos(doslist, minenergy=minenergy, maxenergy=maxenergy, figwidth=figwidth,
                                                figheight=figheight)
        # fig.savefig(figname)



"""
Module for VASP

The Vienna Ab initio Simulation Package (VASP) is a computer program for atomic scale materials modelling,
e.g. electronic structure calculations and quantum-mechanical molecular dynamics, from first principles.

https://www.vasp.at/
"""

from .kpoints import read_kpoints, write_kpoints
from .poscar import read_poscar, write_poscar, write_potcar, get_potcar_info
from .incar import read_incar, write_incar, InputVariables
from .outcar import VaspOutput, read_vasp_stdout
from .vasp import VaspJob, VaspAnalyser
from .doscar import VaspDoscar
from .queue import write_from_queue
from . import task



"""
Utilery definitions
"""

from . import computing
from . import constants
from . import mathematics
from . import metaheuristics
from . import periodic
from . import serializer


# __all__ = filter(lambda s: not s.startswith('_'), dir())



import pychemia
import tempfile
import zipfile
import os


def test_fireball():
    """
    Reading a Fireball output                                    :
    """
    tmpdir = tempfile.mkdtemp()
    path = os.path.abspath('pychemia/test/data/1280.zip')
    zf = zipfile.ZipFile(path)
    oldpwd = os.getcwd()
    os.chdir(tmpdir)
    zf.extract('1280/output.log')
    fo = zf.extract('1280/output.log')
    output = pychemia.code.fireball.read_fireball_stdout(fo)

    # Testing results from reading the Grand Total energy
    assert output['grand_total'][-1] == -31202.46383232
    assert len(output['grand_total']) == 313

    # Cleaning files and directory
    os.remove(fo)
    os.rmdir(tmpdir + os.sep + '1280')
    os.rmdir(tmpdir)
    os.chdir(oldpwd)



#!/usr/bin/env python

import os
import sys
import json
import numpy as np
import matplotlib.pyplot as plt
from scipy.spatial import ConvexHull
import argparse


def spcgrp_props(space_group):
    cst_marker = '.'
    cst_color = '#000000'
    cst_marker_size = 2
    cst_zorder = 2
    cst_fillstyle = 'full'
    if space_group < 3:
        cst_label = 'Triclinic'
    elif space_group < 16:
        cst_label = 'Monoclinic'
        cst_marker = '>'
        cst_marker_size = 3
        cst_zorder = 8
        cst_color = '#00aaaa'
    elif space_group < 75:
        cst_label = 'Orthorhombic'
        cst_marker = 'o'
        cst_marker_size = 3
        cst_zorder = 9
        cst_color = '#0000ff'
        cst_fillstyle = 'none'
    elif space_group < 143:
        cst_label = 'Tetragonal'
        cst_marker = '>'
        cst_marker_size = 4
        cst_zorder = 8
        cst_color = '#00aaaa'
    elif space_group < 168:
        cst_label = 'Trigonal'
        cst_marker = 'v'
        cst_marker_size = 6
        cst_zorder = 8
        cst_color = '#00ff00'
        cst_fillstyle = 'none'
    elif space_group < 195:
        cst_marker = '^'
        cst_label = 'Hexagonal'
        cst_marker_size = 8
        cst_zorder = 8
        cst_color = '#ff00ff'
        cst_fillstyle = 'none'
    else:
        cst_marker = '*'
        cst_label = 'Cubic'
        cst_marker_size = 10
        cst_zorder = 10
        cst_color = '#ff0000'
    return cst_marker, cst_color, cst_label, cst_marker_size, cst_zorder, cst_fillstyle


def formation_energy(energy, x, energy_left, energy_right):
    return energy - (1 - x) * energy_left - x * energy_right


def create_convex(bottom, top, energy_left, energy_right, input, output):

    if not os.path.isfile(input):
        print('File not found %s' % input)

    data = json.load(open(input))
    plt.figure(figsize=(11, 8.5))

    for idata in data:
        if idata['ratio'] == 0:
            if energy_left is None or energy_left > idata['energy_pa']:
                energy_left = idata['energy_pa']
                print('Energy per atom for %2s: %9.3f' % (idata['formula'], energy_left))

        elif idata['ratio'] == 1:
            if energy_right is None or energy_right > idata['energy_pa']:
                energy_right = idata['energy_pa']
                print('Energy per atom for %2s: %9.3f' % (idata['formula'], energy_right))

    if energy_left is None or energy_right is None:
        print('Pure elements not found, formation energy cannot be computed')
        sys.exit(1)

    points = []
    for idata in data:
        x = idata['ratio']
        spcgrp = idata['spcgrp']
        marker, color, lab, m, z, fs = spcgrp_props(spcgrp)
        y = formation_energy(idata['energy_pa'], x, energy_left, energy_right)
        plt.plot(x, y, marker=marker, ms=m, color=color, fillstyle=fs, zorder=z)
        points.append([x, formation_energy(idata['energy_pa'], x, energy_left, energy_right)])

    points.append([0.0, 0.0])
    points.append([1.0, 0.0])

    points = np.array(points)
    hull = ConvexHull(points)

    for simplex in hull.simplices:
        if points[simplex, 1][0] <= 0.0 and points[simplex, 1][1] <= 0.0:
            if points[simplex, 1][0] == 0.0 and points[simplex, 1][1] == 0.0:
                continue
            plt.plot(points[simplex, 0], points[simplex, 1], 'k-', zorder=1)

    ylims = plt.ylim()
    if bottom is None:
        bottom = ylims[0]
    if top is None:
        top = ylims[1]

    for spcgrp in [15, 74, 142, 167, 194, 230]:
        marker, color, lab, m, z, fs = spcgrp_props(spcgrp)
        print('Marker for %12s: %s' % (lab, marker))
        plt.plot(-100, -100, marker, ms=m, fillstyle=fs, color=color, label=lab)

    plt.xlim(-0.05, 1.05)
    print('Limits', bottom, top)
    plt.ylim(bottom, top)
    plt.legend(loc=9, prop={'size': 10}, numpoints=1)
    plt.subplots_adjust(left=0.12, bottom=0.13, right=0.98, top=0.96, wspace=None, hspace=None)
    plt.xlabel(r'Composition balance')
    plt.ylabel(r'Formation Energy [eV]')
    plt.savefig(output)
    return plt.gcf()


if __name__ == "__main__":

    description = """Collect structures from several databases for plotting convex hulls"""

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('-t', '--top',
                        default=None, metavar='ymax', type=float,
                        help='Maximum value of formation energy')
    parser.add_argument('-b', '--bottom',
                        default=None, metavar='ymax', type=float,
                        help='Minimum value of formation energy')
    parser.add_argument('-l', '--left_energy_pa',
                        default=None, metavar='energy_pa', type=float,
                        help='Energy per atom of left specie')
    parser.add_argument('-r', '--right_energy_pa',
                        default=None, metavar='energy_pa', type=float,
                        help='Energy per atom right specie')
    parser.add_argument('-i', '--input',
                        default='convex.json', metavar='convex.json', type=str,
                        help='Input file for the Convex Hull (JSON file)')
    parser.add_argument('-o', '--output',
                        default='convex.pdf', metavar='convex.pdf', type=str,
                        help='Output file (default: convex.pdf)')

    args = parser.parse_args()
    if not os.path.isfile(args.input):
        parser.print_help()
        exit(1)
    print(args)

    create_convex(args.bottom, args.top, args.left_energy_pa, args.right_energy_pa, args.input, args.output)



#!/usr/bin/env python

import os
import pychemia
import tempfile
import shutil
from pychemia.utils.computing import hashfile
from pychemia.db import has_connection


def test_queue():
    """
    Tests (pychemia.db.PyChemiaQueue)                            :
    """
    if not has_connection():
        return

    print("Testing PyChemiaQueue")

    source = 'pychemia/test/data/vasp_01'
    destination = tempfile.mkdtemp()
    print('Destination: %s' % destination)

    st = pychemia.code.vasp.read_poscar(source + os.sep + 'POSCAR')
    print('Structure: \n%s' % st)

    vi = pychemia.code.vasp.read_incar(source + os.sep + 'INCAR')
    print('VASP Input: \n%s' % vi)

    pq = pychemia.db.PyChemiaQueue()

    files = [source + os.sep + 'KPOINTS']
    entry_id = pq.new_entry(structure=st, variables=vi, code='vasp', files=files)

    nfiles = pq.db.fs.files.count()
    print('Number of files: ', nfiles)

    pychemia.code.vasp.write_from_queue(pq, entry_id, destination)

    for i in os.listdir(source):
        assert hashfile(source + os.sep + i) == hashfile(destination + os.sep + i)

    print('Files in source and destination are identical')

    print('Adding the same entry again and testing that the number of files is unchanged')
    entry_id = pq.new_entry(structure=st, variables=vi, code='vasp', files=files)

    assert nfiles == pq.db.fs.files.count()
    print('The number of files remains the same ', nfiles)

    shutil.rmtree(destination)


if __name__ == '__main__':
    test_queue()



import pychemia
import unittest


class KPointsTest(unittest.TestCase):

    def test_optimized_grid(self):
        """
        Tests (pychemia.crystal.kpoints)                             :
        """
        st = pychemia.code.vasp.read_poscar('pychemia/test/data/SbBi/POSCAR')
        kp = pychemia.crystal.KPoints.optimized_grid(st.lattice)
        self.assertEqual(kp.grid, [11, 11, 11])
        self.assertEqual(kp.get_density_of_kpoints(st.lattice), 93126.888547945025)
        kpdict = kp.to_dict
        kp2 = kp.from_dict(kpdict)
        self.assertEqual(kp.grid, kp2.grid)
        kp = pychemia.crystal.KPoints(kmode='path',
                                      kvertices=[[0, 0, 0], [0.5, 0.5, 0], [0.5, 0.5, 0.5], [0, 0, 0]],
                                      intermediates=10)
        self.assertEqual(kp.number_of_kpoints, 40)
        kp = pychemia.crystal.KPoints(kmode='reduced')
        kp.add_kpt([0, 0, 0], weight=0.5)
        self.assertEqual(kp.nkpt, 1)
        kp.add_kpt([0.5, 0.5, 0.5], weight=0.5)
        print(kp)
        self.assertEqual(kp.number_of_kpoints, 2)

        kp.set_kpoints_list([[0.0, 0.0, 0.0], [0.0, 0.5, 0.5], [0.5, 0.0, 0.5], [0.5, 0.5, 0.0], ])
        self.assertEqual(kp.number_of_kpoints, 4)



from __future__ import print_function
import os
import sys
import numpy as np
from pychemia import HAS_MATPLOTLIB


class DensityOfStates:
    """
    Stores the density of states
    It its basically a numpy array with the first column representing energies
    and all the extra columns representing density of states.
    Those several columns could contain partial contributions due to spin and or
    orbital
    """

    def __init__(self, table=None, title=None):
        self._dos = None
        self._min_energy = None
        self._max_energy = None
        self._max_dos = None
        self.title = title
        self.ncols = 1

        if table is not None:
            self._dos = np.array(table)
            self.ncols = self._dos.shape[1] - 1
            self._min_energy = min(table[:, 0])
            self._max_energy = max(table[:, 0])
            self._max_dos = max(table[:, 1])

    @staticmethod
    def read(filename, title=None):
        """
        Reads a file and returns a DensityOFStates object.
        The file could contain concatenated two columns representing
        the desnity of states. However, the energies should be
        repetitivive and the number of values be the same for all the
        sets

        :param filename: (string) the file that contains the density of states
        :param title: (string) customize title
        :return: (DensityOfStates) object
        """
        table = np.loadtxt(filename)
        table = np.array(table)
        if title is None:
            root, ext = os.path.splitext(os.path.basename(filename))
            name = root
        else:
            name = title

        jump = 0
        jumplist = [0]
        nsets = 1
        for iline in range(len(table) - 1):
            if table[iline, 0] != table[iline - jump, 0]:
                print(iline, jump, table[iline, 0], table[iline - jump, 0])
                raise ValueError("No consistency on energy values")
            if table[iline + 1, 0] < table[iline, 0]:
                jump = iline + 1
                jumplist.append(jump)
                nsets += 1

        if nsets > 1:
            jump1 = jumplist[1] - jumplist[0]
            for i in range(1, nsets - 1):
                if jumplist[i + 1] - jumplist[i] != jump1:
                    raise ValueError("No equal jumps")
            table2 = np.zeros((jump1, nsets + 1))
            table2[:, 0] = table[:jump1, 0]
            for i in range(nsets):
                table2[:, i + 1] = table[i * jump1:(i + 1) * jump1, 1]
                assert (np.all(table[i * jump1:(i + 1) * jump1, 0] == table[:jump1, 0]))
        else:
            table2 = table

        dos = DensityOfStates(table=table2, title=name)
        return dos

    @property
    def dos(self):
        return self._dos

    @property
    def energies(self):
        """
        The energy values

        :return: (numpy.ndarray) One-dimensional array of energies
        """
        return self._dos[:, 0]

    @property
    def values(self):
        """
        The density of states values, could be multidimensional

        :return: (numpy.ndarray) Density of states values
        """
        if self.ncols > 1:
            return self._dos[:, range(1, self.ncols + 1)]
        else:
            return self._dos[:, 1]


def plot_one_dos(dosobj, ax=None, horizontal=True, figwidth=16, figheight=12):
    """
    Plot a single density of states, if the values contains
    several dimensions all the dimensions are plotted
    If the axes is no given, a new figure is created and the
    axes is returned

    :param figheight:
    :param figwidth:
    :param dosobj: (DensityOfStates) object
    :param ax: (matplotlib.axes.Axes) object
    :param horizontal: (bool) if the plot is horizontal or

    :return: (matplotlib.figure.Figure, matplotlib.axes.Axes) the (fig, ax) tuple
    """
    if HAS_MATPLOTLIB:
        import matplotlib.pyplot as plt
    else:
        raise NotImplementedError

    if ax is None:
        fig = plt.figure()
        fig.set_figheight(figheight)
        fig.set_figwidth(figwidth)
        ax = fig.add_subplot(111)
        if horizontal:
            ax.set_xlabel('Energy')
        else:
            ax.set_ylabel('Energy')
    else:
        fig = plt.gcf()

    xx = dosobj.energies

    if dosobj.ncols > 1:
        for i in range(dosobj.ncols):
            yy = dosobj.values[:, i]

            if horizontal:
                ax.plot(xx, yy)
            else:
                ax.plot(yy, xx)

    else:
        yy = dosobj.values
        if horizontal:
            ax.plot(xx, yy)
        else:
            ax.plot(yy, xx)

    # fig.savefig('test.pdf')
    return fig, ax


def plot_many_dos(doslist, minenergy=None, maxenergy=None, figwidth=16, figheight=12):
    """
    Plot multiple densities of states

    :param doslist: (list) list of DensityOfStates objects
    :param minenergy: (float) minimal energy to display
    :param maxenergy: (float) maximal energy to display
    :param figheight: (float) Height of figure
    :param figwidth: (float) Width of figure
    """
    import matplotlib.pyplot as plt
    ndos = len(doslist)
    if minenergy is None:
        minenergy = min([min(x.energies) for x in doslist])
    if maxenergy is None:
        maxenergy = max([max(x.energies) for x in doslist])
    minval = sys.float_info.max
    maxval = sys.float_info.min
    for idos in doslist:
        for i in idos.dos:
            if minenergy < i[0] < maxenergy:
                for icol in range(idos.ncols):
                    if i[icol + 1] > maxval:
                        maxval = i[icol + 1]
                    if i[icol + 1] < minval:
                        minval = i[icol + 1]

    fig, ax = plt.subplots(nrows=1, ncols=ndos, sharex=False, sharey=True, squeeze=True)
    fig.set_figwidth(figwidth)
    fig.set_figheight(figheight)
    plt.subplots_adjust(left=0.05, bottom=0.05, right=0.95, top=0.95, wspace=0, hspace=0)
    for i in range(ndos):
        plot_one_dos(doslist[i], ax[i], horizontal=False)
        ax[i].set_xlim(1.1 * minval, 1.1 * maxval)
        ax[i].set_ylim(minenergy, maxenergy)
        ax[i].set_xlabel(doslist[i].title)
        ax[i].spines['bottom'].set_linewidth(10)
        ax[i].spines['left'].set_linewidth(10)
    ax[0].set_ylabel('Energy')
    fig.savefig('test.pdf')
    return fig, ax



#!/usr/bin/env python

import os
import sys
import pychemia

"""
Computes the hardness of a given structure
"""

__author__ = 'Guillermo Avendano Franco'


def helper():
    print("""
Computes the hardness of one or several files

Use:
    hardness.py POSCAR1 [POSCAR2 ...]
""")


if __name__ == '__main__':

    if len(sys.argv) < 2:
        helper()
        sys.exit(1)

    for i in range(1, len(sys.argv)):
        filename = sys.argv[i]
        if not os.path.exists(filename):
            print('Filename not found ', filename)
            continue

        structure = pychemia.code.vasp.read_poscar(filename)
        analysis = pychemia.analysis.StructureAnalysis(structure, supercell=(2, 2, 2))
        print('File : ', filename)
        print(40 * '=' + ' structure ' + 40 * '=')
        print(structure)
        print(40 * '=' + ' structure ' + 40 * '=')

        print('Mass density:', structure.density)

        hardness, r_cutoff, coordination = analysis.hardness(use_laplacian=True, verbose=True)
        print('Hardness : ', hardness)
        print('Cutoff radius :', r_cutoff)



import doctest
import unittest
from pychemia.test.doctest_2to3 import doctest_suite


def broken_function():
    raise Exception('This is broken')


class MyTestCase(unittest.TestCase):
    def test(self):
        """
        DocTests exceptions (pychemia.utils)                         :
        """
        from pychemia.utils.periodic import atomic_number
        with self.assertRaises(Exception) as context:
            atomic_number(['H', u'A'])
        # self.assertTrue(u'Atomic symbol not found' == context.exception)

        from pychemia.utils.computing import read_file
        with self.assertRaises(Exception) as context:
            read_file('/dev/abc')
        # self.assertTrue('Could not open file: /dev/abc' in context.exception)

        from pychemia.utils.computing import get_float
        with self.assertRaises(Exception) as context:
            get_float('3i')
            # self.assertTrue("Could not convert '3i' into a float number" in context.exception)


def test_periodic():
    """
    DocTests (pychemia.utils.periodic)                           :
    """
    import pychemia.utils.periodic
    dt = doctest.testmod(pychemia.utils.periodic, verbose=True)
    assert dt.failed == 0


def test_mathematics():
    """
    DocTests (pychemia.utils.mathematics)                        :
    """
    import pychemia.utils.mathematics
    dt = doctest.testmod(pychemia.utils.mathematics, verbose=True)
    assert dt.failed == 0


def test_computing():
    """
    DocTests (pychemia.utils.computing)                          :
    """
    import pychemia.utils.computing
    suite = unittest.TestSuite()
    suite.addTest(doctest_suite(pychemia.utils.computing))
    runner = unittest.TextTestRunner(verbosity=1)
    result = runner.run(suite)
    assert result.wasSuccessful()


if __name__ == "__main__":
    unittest.main(defaultTest='test_computing')
    unittest.main()



#!/usr/bin/env python
from __future__ import print_function
import os
import getopt
import json
import multiprocessing
import random
import socket
import sys
import time
from bson.objectid import ObjectId
import pychemia
from pychemia.utils.computing import get_int

__author__ = 'Guillermo Avendano-Franco'


def usage(name):
    print("""
NAME
    %s

DESCRIPTION
    PyChemia Server intended to be run as a daemon on the frontend


OPTIONS

    --help, -h
        Return information on the options and use of this script

    --dbsettings_file, -d <string>
        The PyChemiaQueue database that will be used as source for jobs

    --port, -p <int> (Default: 10000)
        The UDP port that will be used for transfer data with clients

    --workdir, w <string>
        Working directory were files will be created and jobs runned

    --ip, -i <string>
        IP address to bind the server

""" % os.path.basename(name))


def get_database(dbsettings):
    if 'host' not in dbsettings:
        dbsettings['host'] = 'localhost'
    if 'port' not in dbsettings:
        dbsettings['port'] = 27017
    if 'ssl' not in dbsettings:
        dbsettings['ssl'] = False

    if 'user' not in dbsettings:
        pcq = pychemia.db.PyChemiaQueue(name=dbsettings['name'], host=dbsettings['host'], port=dbsettings['port'],
                                        ssl=dbsettings['ssl'])
    else:
        pcq = pychemia.db.PyChemiaQueue(name=dbsettings['name'], host=dbsettings['host'], port=dbsettings['port'],
                                        user=dbsettings['user'], passwd=dbsettings['passwd'], ssl=dbsettings['ssl'])
    return pcq


def listener(dbsettings, ip, port, workdir):
    # Create a TCP/IP socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    # Bind the socket to the port
    server_address = (ip, port)
    print('starting up on %s port %s' % server_address, file=sys.stderr)
    sock.bind(server_address)

    pcq = get_database(dbsettings)

    while True:
        print('\nwaiting to receive message', file=sys.stderr)
        data, address = sock.recvfrom(4096)

        print('received %s bytes from %s' % (len(data), address), file=sys.stderr)
        print(data, file=sys.stderr)

        if data == 'COUNT':
            ans = str(pcq.db.pychemia_entries.count())
            sent = sock.sendto(ans, address)
            print('sent %s bytes back to %s' % (sent, address), file=sys.stderr)
        if data == 'WORKDIR':
            sent = sock.sendto(workdir, address)
            print('sent %s bytes back to %s' % (sent, address), file=sys.stderr)
        if data == 'GET':
            # Selectiong an entry not submitted for execution
            entry = pcq.db.pychemia_entries.find_one({'meta.submitted': False}, {'_id': 1})
            if entry is not None:
                entry_id = entry['_id']
                pcq.db.pychemia_entries.update({'_id': entry_id}, {'$set': {'meta.submitted': True}})
                deploy(entry_id, pcq, workdir)
            else:
                entry_id = ''
            sent = sock.sendto(str(entry_id), address)
            print('sent %s bytes back to %s' % (sent, address), file=sys.stderr)
        if data.startswith('FINISHED:'):
            entry_id = ObjectId(data.split(':')[1])
            collect(entry_id, pcq, workdir)
            pcq.db.pychemia_entries.update({'_id': entry_id}, {'$set': {'meta.finished': True}})
            sent = sock.sendto('OK', address)
            print('sent %s bytes back to %s' % (sent, address), file=sys.stderr)


def deploy(entry_id, pychemia_queue, basedir):
    workdir = os.path.abspath(basedir) + os.sep + str(entry_id)
    if not os.path.exists(workdir):
        os.mkdir(workdir)
    pychemia_queue.write_input_files(entry_id, destination=workdir)

    entry = pychemia_queue.db.pychemia_entries.find_one({'_id': entry_id}, {'job': 1})
    job_settings = entry['job']
    print(job_settings)

    if len(job_settings) > 0:
        wf = open(workdir + os.sep + 'job.json', 'w')
        json.dump(job_settings, wf)
        wf.close()

    st = pychemia_queue.get_input_structure(entry_id)
    st.save_json(workdir + os.sep + 'structure.json')

    inp = pychemia_queue.get_input_variables(entry_id)
    if inp is not None:
        wf = open(workdir + os.sep + 'input.json', 'w')
        json.dump(inp, wf)
        wf.close()


def collect(entry_id, pychemia_queue, workdir):
    destination = os.path.abspath(workdir) + os.sep + str(entry_id)
    if os.path.isfile(destination + os.sep + 'results.json'):
        results = json.load(open(destination + os.sep + 'results.json'))
        pychemia_queue.db.pychemia_entries.update({'_id': entry_id}, {'$set': {'output': results}})


def main(argv):
    try:
        opts, args = getopt.getopt(argv[1:], "hd:p:i:w:", ["help", "dbsettings_file=", "port=", "ip=", 'workdir='])
    except getopt.GetoptError:
        print('Error with options')
        usage(argv[0])
        sys.exit(2)

    if len(opts) == 0:
        usage(argv[0])
        sys.exit(2)

    # Default Values
    workdir = None
    dbsettings_file = None
    port = None
    ip = None

    for opt, arg in opts:

        if opt in ("-h", "--help"):
            usage(argv[0])
            sys.exit()
        elif opt in ("-w", "--workdir"):
            workdir = arg
        elif opt in ("-d", "--dbsettings_file"):
            dbsettings_file = arg
        elif opt in ("-p", "--port"):
            port = get_int(arg)
        elif opt in ("-i", "--ip"):
            ip = arg

    if workdir is None:
        print('Missing workdir')
        usage(argv[0])
        sys.exit()
    if not os.path.isdir(workdir):
        os.mkdir(workdir)

    print('dbsettings_file:', dbsettings_file)
    if dbsettings_file is None or not os.path.exists(dbsettings_file):
        print('Missing dbsettings')
        usage(argv[0])
        sys.exit()
    else:
        dbsettings = json.load(open(dbsettings_file))

    if ip is None:
        ip = socket.gethostbyname(socket.gethostname())

    print('PyChemia Server binded to IP: ', ip)

    pcq = get_database(dbsettings)
    print('Connected with database, number of entries: ', pcq.db.pychemia_entries.count())

    if port is not None:
        print('Using port : %s' % port)
        p = multiprocessing.Process(target=listener, args=(dbsettings, ip, port, workdir))
        p.start()
    else:
        old_port = 0
        p = None
        while True:
            lt = time.localtime()
            random.seed(lt.tm_yday * 24 + lt.tm_hour)
            port = random.randint(10000, 20000)
            if port != old_port:
                if p is not None and p.is_alive():
                    print('Terminating listener on port : %s' % old_port)
                    p.terminate()
                print('Using port : %s' % port)
                p = multiprocessing.Process(target=listener, args=(dbsettings, ip, port, workdir))
                p.start()
                old_port = port
            time.sleep(300)


if __name__ == "__main__":
    main(sys.argv)



"""
Routines anc Classes to manipulate Octopus Inputs
"""

import os as _os
import gzip as _gzip
import numpy as _np
import pychemia


def execute(basedir, num_threads=1, num_procs=2):
    """
    Call octopus in parallel with a given number of threads and
    MPI process

    :param basedir: (str) Working directory for execution
    :param num_threads: (int) Number of OpenMP Threads to use
    :param num_procs: (int) Number of MPI processes to create
    """
    wfile = open('script.sh', 'w')
    wfile.write('export OMP_NUM_THREADS=' + str(num_threads) + '\n')
    wfile.write('mpirun -np ' + str(num_procs) + ' octopus_mpi')
    wfile.close()
    pychemia.runner.execute(basedir, 'bash', 'script.sh')


def get_vars():
    """
    Get a list of all the variables in octopus
    """
    data = open(_os.path.dirname(pychemia.__path__[0]) + '/pychemia/octopus/octopus_variables.conf')
    oct_variables = {}

    for line in data.readlines():
        line = line.strip()
        if len(line) == 0:
            continue
        elif line[0] == '[':
            section = line.strip()[1:-1]
        elif len(line.strip()) > 0:
            if section not in oct_variables.keys():
                oct_variables[section] = [line]
            else:
                oct_variables[section].append(line)
    return oct_variables


class OpenDX:
    """
    Manipulate the OpenDX file generated by octopus
    """

    def __init__(self, filename):
        """
        Creates an OpenDX file generated for octopus
        """
        rfile = _gzip.open(filename)
        data = rfile.readlines()
        rfile.close()
        del rfile
        self.nsize = _np.array([int(x) for x in data[0].split()[-3:]])
        self.origin = _np.array([float(x) for x in data[1].split()[-3:]])
        self.delta = _np.array([[float(x.split()[i]) for i in [1, 2, 3]] for x in data[2:5]])
        self.field = _np.array(map(float, data[7:_np.prod(self.nsize) + 7]))
        self.field = self.field.reshape(tuple(self.nsize))
        del data

    def integrate_x(self):
        """
        Compute the Integral in the X direction
        """
        xvalue = _np.arange(self.origin[0], self.origin[0] + self.delta[0, 0] * self.nsize[0], self.delta[0, 0])
        yvalue = _np.sum(_np.sum(self.field, axis=1), axis=1)
        return xvalue, yvalue


class InputVariables:
    """
    Manipulate an octopus input file
    """

    def __init__(self, filename=None):
        """
        Converts a given octopus input file
        into a dictionary where the keys are
        variable names and the values are scalars
        or list if the variable is a block
        """
        key = ''
        self.variables = {}
        if filename is None:
            return

        rfile = open(filename)

        multivalue = False
        values = None
        for line in rfile.readlines():

            if multivalue:
                if line.strip()[0] == '%' and len(line.strip()) == 1:
                    self.variables[key] = values
                    multivalue = False
                    values = None
                elif values is None:
                    values = [x.strip() for x in line.split('|')]
                else:
                    if type(values[0]) != list:
                        values = [values]
                    values.append([x.strip() for x in line.split('|')])
            elif line == '\n':
                continue
            elif line.strip()[0] == '#':
                continue
            elif len(line.split('=')) == 2:
                key = line.split('=')[0].strip()
                value = line.split('=')[1].strip()
                self.variables[key] = value
            elif line.strip()[0] == '%' and len(line.strip()) > 1:
                key = line.strip()[1:]
                multivalue = True
            else:
                print('Line not parsed:', line)

        rfile.close()

    def write(self, filename):
        """
        Write an input dictionary into a file
        the variables are sort by kind and written
        in their respective place

        :param filename: (str) Filename for the octopus input that will be created
        """
        wfile = open(filename, 'w')
        wfile.write(self.__str__())
        wfile.close()

    def __str__(self):
        """
        Creates an string representation of
        the input as it will be written
        in a file
        """
        oct_vars = get_vars()
        octvars = self.variables.keys()

        oct_str = ""
        # Writing the known groups
        for group in sorted(oct_vars.keys()):
            use = 0
            for j in sorted(oct_vars[group]):
                if j in octvars:
                    if use == 0:
                        oct_str += '\n#' + 60 * '-' + '\n'
                        oct_str += '#' + 3 * ' ' + ' ' + group + '\n'
                        oct_str += '#' + 60 * '-' + '\n\n'
                        use = 1

                    oct_str += _write_key(j, self.variables[j])
                    octvars.remove(j)

        oct_def = ""
        if len(octvars) > 0:
            # Writing others
            oct_def += '\n#' + 60 * '-' + '\n'
            oct_def += '#' + 3 * ' ' + ' ' + 'Definitions' + '\n'
            oct_def += '#' + 60 * '-' + '\n\n'
            print('Non grouped variables:')

            for j in octvars:
                oct_def += _write_key(j, self.variables[j])
                print(' * ', j)

        return oct_def + oct_str


def _write_key(key, value):
    """
    Write in the proper format the key and the value in
    a octopus input file
    """
    oct_str = ""
    keylen = 20

    if isinstance(value, int) or isinstance(value, float):
        oct_str = oct_str + (key.ljust(keylen) + " = " + str(value) + '\n')
    elif isinstance(value, str):
        if (key == 'XYZCoordinates' or key == 'XYZVelocities') and '"' not in value:
            oct_str = oct_str + (key.ljust(keylen) + ' = "' + value + '"\n')
        else:
            oct_str = oct_str + (key.ljust(keylen) + " = " + value + '\n')

    elif isinstance(value, list):
        oct_str += '\n%' + key + "\n"
        for itable in range(len(value)):
            if isinstance(value[itable], list):
                for jtable in range(len(value[itable])):
                    oct_str += str(value[itable][jtable]).ljust(10)

                    if jtable < len(value[itable]) - 1:
                        oct_str += " | "
                    if jtable == len(value[itable]) - 1 and itable < len(value) - 1:
                        oct_str += "\n"
            else:
                oct_str += str(value[itable])
                if itable < len(value) - 1:
                    oct_str += " | "
        oct_str += "\n%\n\n"

    return oct_str



__author__ = 'Guillermo Avendano-Franco'


def test_good():
    """
    Simple test import pychemia                                  :
    """
    from pychemia import pcm_log
    pcm_log.debug('DEBUGGING')
    pass



#!/usr/bin/env python

import os
import sys
import logging
import pychemia
from pychemia.external.pymatgen import *
from pychemia import pcm_log


def help_info():
    print(""" Create or Update a Database from CIFs

   Use:

       CreateDatabaseFromCIFs.py --dbname 'MongoDB Database name' --path 'Directory with CIFs'
                                [--host localhost ] [--port 27017] [--ssl]
                                [--user None ] [--passwd None]

   """)


version = 0.1

if __name__ == '__main__':

    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger('pychemia')
    logger.addHandler(logging.NullHandler())
    logger.setLevel(logging.DEBUG)

    # Script starts from here
    if len(sys.argv) < 2:
        help_info()
        sys.exit(1)

    dbname = None
    host = 'localhost'
    port = 27017
    user = None
    passwd = None
    path = None
    ssl = False

    for i in range(1, len(sys.argv)):
        if sys.argv[i].startswith('--'):
            option = sys.argv[i][2:]
            # fetch sys.argv[1] but without the first two characters
            if option == 'version':
                print(version)
                sys.exit()
            elif option == 'help':
                help_info()
                sys.exit()
            elif option == 'dbname':
                dbname = sys.argv[i + 1]
            elif option == 'host':
                host = sys.argv[i + 1]
            elif option == 'port':
                dbname = int(sys.argv[i + 1])
            elif option == 'user':
                user = sys.argv[i + 1]
            elif option == 'passwd':
                passwd = sys.argv[i + 1]
            elif option == 'path':
                path = sys.argv[i + 1]
            elif option == 'ssl':
                ssl = True
            else:
                print('Unknown option. --' + option)

    if dbname is None:
        help_info()
        sys.exit(1)

    db_settings = {'name': dbname, 'host': host, 'port': port, 'ssl': ssl}
    if user is not None:
        if passwd is None:
            raise ValueError('Password is mandatory if user is entered')
        db_settings['user'] = user
        db_settings['passwd'] = passwd

    pcdb = pychemia.db.get_database(db_settings)

    cifs = [x for x in os.listdir(path) if x[-3:] == 'cif']

    for i in cifs:
        pcm_log.info('Reading CIF         : %s' % i)
        structs = cif2structure(path + os.sep + i)
        pcm_log.info('Number of structures: %d' % len(structs))
        for j in structs:
            if j.is_perfect:
                pcm_log.info('Composition         : %s' % str(j.composition))
                pcdb.insert(structure=j)
            else:
                pcm_log.info('DISCARDED: Structure is not perfect')



#!/usr/bin/env python

import os
import re
import shutil
import argparse
import logging
import numpy as np
import subprocess
from pychemia import pcm_log, Structure
from pychemia.analysis import StructureAnalysis
from pychemia.code.vasp import write_poscar, read_poscar
from pychemia.code.vasp.task import IonRelaxation
from pychemia.db import get_database
from pychemia.evaluator import DirectEvaluator
from pychemia.utils.serializer import generic_serializer
from pychemia.utils.periodic import atomic_number
from pychemia.code.vasp.task import IonRelaxation2
from pychemia.code.dftb.task import Relaxation
from pychemia.code.dftb import read_detailed_out


def worker_maise(db_settings, entry_id, workdir, relaxator_params):
    """
    Relax and return evaluate the energy of the structure stored with identifier 'entry_id'
     using the MAISE code

    :param db_settings: (dict) Dictionary of DB parameters needed to create a PyChemiaDB object
    :param entry_id: MongoDB identifier of one entry of the database created from db_settings
    :param workdir: (str) Working directory where input and output from MAISE is written
    :param relaxator_params: (dict) Arguments needed to control the relaxation using MAISE
                                Arguments are store as keys and they include:
                                'target_forces' : Used to defined the tolerance to consider one candidate as relaxed.
                                'source_dir': Directory with executable maise and directory INI
    :return:
    """
    max_ncalls = 6
    pcdb = get_database(db_settings)
    target_forces = relaxator_params['target_forces']
    source_dir = relaxator_params['source_dir']

    pcm_log.info('[%s]: Starting relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    if pcdb.is_locked(entry_id):
        return
    else:
        pcdb.lock(entry_id)
    structure = pcdb.get_structure(entry_id)
    status = pcdb.get_dicts(entry_id)[2]

    if 'ncalls' in status and status['ncalls'] > 0:
        ncalls = status['ncalls'] + 1
        print('ncalls = ', status['ncalls'])
    else:
        ncalls = 1
    print('Verifing initial structure...')
    while np.min(structure.distance_matrix()+(np.eye(structure.natom)*5)) < 1.9:
        print('ERROR: Bad initial guess, two atoms are to close. Creating new random structure for id: %s' %
              str(entry_id))
        write_poscar(structure, workdir + os.sep + 'Fail_initial_POSCAR')  # WIH
        structure = Structure.random_cell(structure.composition)

    write_poscar(structure, workdir + os.sep + 'POSCAR')
    if not os.path.exists(workdir + os.sep + 'setup') and ncalls == 1:     # WIH
        print('First run.')  # WIH
        #   print('Verifying that everything runs smoothly') # WIH
        print(workdir + os.sep + 'setup')
        shutil.copy2(source_dir + os.sep + 'setup_1', workdir + os.sep + 'setup')   # WIH
    elif ncalls > 1:  # WIH
        shutil.copy2(source_dir + os.sep + 'setup_2', workdir + os.sep + 'setup')   # WIH
    if not os.path.exists(workdir + os.sep + 'INI'):
        os.symlink(source_dir + os.sep + 'INI', workdir + os.sep + 'INI')
    if not os.path.exists(workdir + os.sep + 'maise'):
        os.symlink(source_dir + os.sep + 'maise', workdir + os.sep + 'maise')

    # Get the Current Working Directory
    # cwd = os.getcwd()

    # Move to the actual directory where maise will run
    os.chdir(workdir)

    wf = open('maise.stdout', 'w')
    subprocess.call(['./maise'], stdout=wf)
    wf.close()

    if os.path.isfile('OSZICAR'):
        energies = np.loadtxt('OSZICAR')
    else:
        energies = None

    forces = None
    stress = None
    stress_kb = None
    if os.path.isfile('OUTCAR'):
        rf = open('OUTCAR', 'r')
        data = rf.read()

        pos_forces = re.findall(r'TOTAL-FORCE \(eV/Angst\)\s*-*\s*([-.\d\s]+)\s+-{2}', data)
        pos_forces = np.array([x.split() for x in pos_forces], dtype=float)

        if len(pos_forces) > 0 and len(pos_forces[-1]) % 7 == 0:
            pos_forces.shape = (len(pos_forces), -1, 7)
            forces = pos_forces[:, :, 3:6]
            # positions = pos_forces[:, :, :3]
        else:
            print('Forces and Positions could not be parsed : ', pos_forces.shape)
            print('pos_forces =\n%s ' % pos_forces)

        str_stress = re.findall('Total([.\d\s-]*)in', data)
        if len(str_stress) == 2:
            stress = np.array([[float(y) for y in x.split()] for x in str_stress])
        str_stress = re.findall('in kB([.\d\s-]*)energy', data)
        if len(str_stress) == 2:
            stress_kb = np.array([[float(y) for y in x.split()] for x in str_stress])

    create_new = False
    if not os.path.isfile('CONTCAR') or os.path.getsize("CONTCAR") == 0:
        create_new = True
        print('CONTCAR not found in entry: %s' % str(entry_id))
        i = 1
        while True:
            if not os.path.isfile('POSCAR-failed-%03s' % str(i)):
                os.rename('POSCAR', 'POSCAR-failed-%03s' % str(i))
                break
            else:
                i += 1
    else:
        new_structure = read_poscar('CONTCAR')
        # min_dist = np.min(new_structure.distance_matrix+np.ones((new_structure.natom,new_structure.natom)))
    min_dist = np.min(new_structure.distance_matrix()+(np.eye(new_structure.natom)*5))   # WIH
    print('Minimal distance= %8.7f' % min_dist)   # WIH

    if min_dist < 2.0:
        print('ERROR: MAISE finished with and structure with distances too close:', entry_id)  # WIH
        write_poscar(new_structure, workdir + os.sep + 'Collapsed_CONTCAR')  # WIH
        create_new = True   # WIH

    if create_new:
        new_structure = Structure.random_cell(structure.composition)
        ncalls = 0    # WIH

    if ncalls > max_ncalls:
        print('WARNING: Too many calls to MAISE and no relaxation succeeded, replacing structure: ', entry_id)    # WIH
        new_structure = Structure.random_cell(structure.composition)
        pcdb.entries.update({'_id': entry_id}, {'$set': {'status.ncalls': 0}})
        create_new = True
    else:
        pcdb.entries.update({'_id': entry_id}, {'$set': {'status.ncalls': ncalls}})
    pcdb.update(entry_id, structure=new_structure, properties={})

    # if not create_new and energies is not None and forces is not None and stress is not None:
    if energies is not None and forces is not None and stress is not None:

        te = energies[1]
        pcdb.entries.update({'_id': entry_id},
                            {'$set': {'status.relaxation': 'succeed',
                                      'status.target_forces': target_forces,
                                      'properties.initial_forces': generic_serializer(forces[0]),
                                      'properties.initial_stress': generic_serializer(stress[0]),
                                      'properties.initial_stress_kB': generic_serializer(stress_kb[0]),
                                      'properties.forces': generic_serializer(forces[1]),
                                      'properties.stress': generic_serializer(stress[1]),
                                      'properties.stress_kB': generic_serializer(stress_kb[1]),
                                      'properties.energy': te,
                                      'properties.energy_pa': te / new_structure.natom,
                                      'properties.energy_pf': te / new_structure.get_composition().gcd}})

    for ifile in ['POSCAR', 'CONTCAR', 'setup', 'OUTCAR', 'maise.stdout', 'list.dat']:
        if not os.path.exists(ifile):
            wf = open(ifile, 'w')
            wf.write('')
            wf.close()
        n = 1
        while True:
            if os.path.exists(ifile + ('_%03d' % n)):
                n += 1
            else:
                break
        os.rename(ifile, ifile+('_%03d' % n))

    pcm_log.info('[%s]: Unlocking the entry' % str(entry_id))
    pcdb.unlock(entry_id)


def worker_vasp(db_settings, entry_id, workdir, relaxator_params):
    pcdb = get_database(db_settings)
    target_forces = relaxator_params['target_forces']
    pcm_log.info('[%s]: Starting relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    if pcdb.is_locked(entry_id):
        return
    else:
        pcdb.lock(entry_id)
    structure = pcdb.get_structure(entry_id)
    structure = structure.scale()
    print('relaxator_params', relaxator_params)
    relaxer = IonRelaxation(structure, workdir=workdir, target_forces=target_forces, waiting=False,
                            binary=relaxator_params['binary'], encut=1.3, kp_grid=None, kp_density=1E4,
                            relax_cell=True, max_calls=10)
    print('relaxing on:', relaxer.workdir)
    relaxer.run(relaxator_params['nmpiparal'])
    pcm_log.info('[%s]: Finished relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    filename = workdir + os.sep + 'OUTCAR'
    if os.path.isfile(filename):

        forces, stress, total_energy = relaxer.get_forces_stress_energy()

        if forces is not None:
            magnitude_forces = np.apply_along_axis(np.linalg.norm, 1, forces)
            print('Forces: Max: %9.3e Avg: %9.3e' % (np.max(magnitude_forces), np.average(magnitude_forces)))
            print('Stress: ', np.max(np.abs(stress.flatten())))

        if forces is None:
            pcm_log.error('No forces found on %s' % filename)
        if stress is None:
            pcm_log.error('No stress found on %s' % filename)
        if total_energy is None:
            pcm_log.error('No total_energy found on %s' % filename)

        new_structure = relaxer.get_final_geometry()

        if forces is not None and stress is not None and total_energy is not None and new_structure is not None:
            pcm_log.info('[%s]: Updating properties' % str(entry_id))
            pcdb.update(entry_id, structure=new_structure)
            te = total_energy
            pcdb.entries.update({'_id': entry_id},
                                {'$set': {'status.relaxation': 'succeed',
                                          'status.target_forces': target_forces,
                                          'properties.forces': generic_serializer(forces),
                                          'properties.stress': generic_serializer(stress),
                                          'properties.energy': te,
                                          'properties.energy_pa': te / new_structure.natom,
                                          'properties.energy_pf': te / new_structure.get_composition().gcd}})

            # Fingerprint
            # Update the fingerprints only if the two structures are really different
            diffnatom = structure.natom != new_structure.natom
            diffcell = np.max(np.abs((structure.cell - new_structure.cell).flatten()))
            diffreduced = np.max(np.abs((structure.reduced - new_structure.reduced).flatten()))
            if diffnatom != 0 or diffcell > 1E-7 or diffreduced > 1E-7:
                analysis = StructureAnalysis(new_structure, radius=50)
                x, ys = analysis.fp_oganov(delta=0.01, sigma=0.01)
                fingerprint = {'_id': entry_id}
                for k in ys:
                    atomic_number1 = atomic_number(new_structure.species[k[0]])
                    atomic_number2 = atomic_number(new_structure.species[k[1]])
                    pair = '%06d' % min(atomic_number1 * 1000 + atomic_number2,
                                        atomic_number2 * 1000 + atomic_number1)
                    fingerprint[pair] = list(ys[k])

                if pcdb.db.fingerprints.find_one({'_id': entry_id}) is None:
                    pcdb.db.fingerprints.insert(fingerprint)
                else:
                    pcdb.db.fingerprints.update({'_id': entry_id}, fingerprint)
            else:
                pcm_log.debug('Original and new structures are very similar.')
                pcm_log.debug('Max diff cell: %10.3e' % np.max(np.absolute((structure.cell -
                                                                            new_structure.cell).flatten())))
                if structure.natom == new_structure.natom:
                    pcm_log.debug('Max diff reduced coordinates: %10.3e' %
                                  np.max(np.absolute((structure.reduced - new_structure.reduced).flatten())))

        else:
            pcdb.entries.update({'_id': entry_id}, {'$set': {'status.relaxation': 'failed'}})
            pcm_log.error('Bad data after relaxation. Tagging relaxation as failed')
    else:
        pcm_log.error('ERROR: File not found %s' % filename)
    pcm_log.info('[%s]: Unlocking the entry' % str(entry_id))
    pcdb.unlock(entry_id)


def worker_dftb(db_settings, entry_id, workdir, target_forces, relaxator_params):
    pcdb = get_database(db_settings)
    pcm_log.info('[%s]: Starting relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    if pcdb.is_locked(entry_id):
        return
    else:
        pcdb.lock(entry_id)
    structure = pcdb.get_structure(entry_id)
    structure = structure.scale()
    if 'forced' in relaxator_params:
        forced = relaxator_params['forced']
    else:
        forced = True

    relaxer = Relaxation(structure, relaxator_params=relaxator_params, workdir=workdir,
                         target_forces=target_forces, waiting=True, forced=forced)
    relaxer.run()
    pcm_log.info('[%s]: Finished relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    filename = workdir + os.sep + 'detailed.out'
    if os.path.isfile(filename):
        detailed = read_detailed_out(filename)
        if 'max_force' in detailed:
            max_force = detailed['max_force']
        else:
            max_force = target_forces

        if 'max_deriv' in detailed:
            max_deriv = detailed['max_deriv']
        else:
            max_deriv = target_forces

        forces, stress, total_energy = relaxer.get_forces_stress_energy()

        if forces is not None:
            magnitude_forces = np.apply_along_axis(np.linalg.norm, 1, forces)
            print('Forces: Max: %9.3e Avg: %9.3e' % (np.max(magnitude_forces), np.average(magnitude_forces)))
            print('Stress: ', np.max(np.abs(stress.flatten())))

        if forces is None:
            pcm_log.error('No forces found on %s' % filename)
        if stress is None:
            pcm_log.error('No stress found on %s' % filename)
        if total_energy is None:
            pcm_log.error('No total_energy found on %s' % filename)

        new_structure = relaxer.get_final_geometry()

        if forces is not None and stress is not None and total_energy is not None and new_structure is not None:
            pcm_log.info('[%s]: Updating properties' % str(entry_id))
            pcdb.update(entry_id, structure=new_structure)
            te = total_energy
            pcdb.entries.update({'_id': entry_id},
                                {'$set': {'status.relaxation': 'succeed',
                                          'status.target_forces': target_forces,
                                          'properties.forces': generic_serializer(forces),
                                          'properties.stress': generic_serializer(stress),
                                          'properties.energy': te,
                                          'properties.max_force': max_force,
                                          'properties.max_deriv': max_deriv,
                                          'properties.energy_pa': te / new_structure.natom,
                                          'properties.energy_pf': te / new_structure.get_composition().gcd}})

            # Fingerprint
            # Update the fingerprints only if the two structures are really different
            diffnatom = structure.natom != new_structure.natom
            diffcell = np.max(np.abs((structure.cell - new_structure.cell).flatten()))
            diffreduced = np.max(np.abs((structure.reduced - new_structure.reduced).flatten()))
            if diffnatom != 0 or diffcell > 1E-7 or diffreduced > 1E-7:
                analysis = StructureAnalysis(new_structure, radius=50)
                x, ys = analysis.fp_oganov(delta=0.01, sigma=0.01)
                fingerprint = {'_id': entry_id}
                for k in ys:
                    atomic_number1 = atomic_number(new_structure.species[k[0]])
                    atomic_number2 = atomic_number(new_structure.species[k[1]])
                    pair = '%06d' % min(atomic_number1 * 1000 + atomic_number2,
                                        atomic_number2 * 1000 + atomic_number1)
                    fingerprint[pair] = list(ys[k])

                if pcdb.db.fingerprints.find_one({'_id': entry_id}) is None:
                    pcdb.db.fingerprints.insert(fingerprint)
                else:
                    pcdb.db.fingerprints.update({'_id': entry_id}, fingerprint)
            else:
                pcm_log.debug('Original and new structures are very similar.')
                pcm_log.debug('Max diff cell: %10.3e' % np.max(np.absolute((structure.cell -
                                                                            new_structure.cell).flatten())))
                if structure.natom == new_structure.natom:
                    pcm_log.debug('Max diff reduced coordinates: %10.3e' %
                                  np.max(np.absolute((structure.reduced - new_structure.reduced).flatten())))

        else:
            pcdb.entries.update({'_id': entry_id}, {'$set': {'status.relaxation': 'failed'}})
            pcm_log.error('Bad data after relaxation. Tagging relaxation as failed')
    else:
        pcm_log.error('ERROR: File not found %s' % filename)
    pcm_log.info('[%s]: Unlocking the entry' % str(entry_id))
    pcdb.unlock(entry_id)


def worker(db_settings, entry_id, workdir, target_forces, relaxator_params):
    pcdb = get_database(db_settings)
    pcm_log.info('[%s]: Starting relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    if pcdb.is_locked(entry_id):
        return
    else:
        pcdb.lock(entry_id)
    structure = pcdb.get_structure(entry_id)
    structure = structure.scale()
    print('relaxator_params', relaxator_params)
    relaxer = IonRelaxation2(structure, workdir=workdir, target_forces=target_forces, waiting=False,
                             binary=relaxator_params['binary'], encut=1.3, kp_grid=None, kp_density=1E4,
                             relax_cell=True)
    print('relaxing on:', relaxer.workdir)
    relaxer.run(relaxator_params['nmpiparal'])
    pcm_log.info('[%s]: Finished relaxation. Target forces: %7.3e' % (str(entry_id), target_forces))

    filename = workdir + os.sep + 'OUTCAR'
    if os.path.isfile(filename):

        forces, stress, total_energy = relaxer.get_forces_stress_energy()

        if forces is not None:
            magnitude_forces = np.apply_along_axis(np.linalg.norm, 1, forces)
            print('Forces: Max: %9.3e Avg: %9.3e' % (np.max(magnitude_forces), np.average(magnitude_forces)))
            print('Stress: ', np.max(np.abs(stress.flatten())))

        if forces is None:
            pcm_log.error('No forces found on %s' % filename)
        if stress is None:
            pcm_log.error('No stress found on %s' % filename)
        if total_energy is None:
            pcm_log.error('No total_energy found on %s' % filename)

        new_structure = relaxer.get_final_geometry()

        if forces is not None and stress is not None and total_energy is not None and new_structure is not None:
            pcm_log.info('[%s]: Updating properties' % str(entry_id))
            pcdb.update(entry_id, structure=new_structure)
            te = total_energy
            pcdb.entries.update({'_id': entry_id},
                                {'$set': {'status.relaxation': 'succeed',
                                          'status.target_forces': target_forces,
                                          'properties.forces': generic_serializer(forces),
                                          'properties.stress': generic_serializer(stress),
                                          'properties.energy': te,
                                          'properties.energy_pa': te / new_structure.natom,
                                          'properties.energy_pf': te / new_structure.get_composition().gcd}})

            # Fingerprint
            # Update the fingerprints only if the two structures are really different
            diffnatom = structure.natom != new_structure.natom
            diffcell = np.max(np.abs((structure.cell - new_structure.cell).flatten()))
            diffreduced = np.max(np.abs((structure.reduced - new_structure.reduced).flatten()))
            if diffnatom != 0 or diffcell > 1E-7 or diffreduced > 1E-7:
                analysis = StructureAnalysis(new_structure, radius=50)
                x, ys = analysis.fp_oganov(delta=0.01, sigma=0.01)
                fingerprint = {'_id': entry_id}
                for k in ys:
                    atomic_number1 = atomic_number(new_structure.species[k[0]])
                    atomic_number2 = atomic_number(new_structure.species[k[1]])
                    pair = '%06d' % min(atomic_number1 * 1000 + atomic_number2,
                                        atomic_number2 * 1000 + atomic_number1)
                    fingerprint[pair] = list(ys[k])

                if pcdb.db.fingerprints.find_one({'_id': entry_id}) is None:
                    pcdb.db.fingerprints.insert(fingerprint)
                else:
                    pcdb.db.fingerprints.update({'_id': entry_id}, fingerprint)
            else:
                pcm_log.debug('Original and new structures are very similar.')
                pcm_log.debug('Max diff cell: %10.3e' % np.max(np.absolute((structure.cell -
                                                                            new_structure.cell).flatten())))
                if structure.natom == new_structure.natom:
                    pcm_log.debug('Max diff reduced coordinates: %10.3e' %
                                  np.max(np.absolute((structure.reduced - new_structure.reduced).flatten())))

        else:
            pcdb.entries.update({'_id': entry_id}, {'$set': {'status.relaxation': 'failed'}})
            pcm_log.error('Bad data after relaxation. Tagging relaxation as failed')
    else:
        pcm_log.error('ERROR: File not found %s' % filename)
    pcm_log.info('[%s]: Unlocking the entry' % str(entry_id))
    pcdb.unlock(entry_id)


def is_evaluated(pcdb, entry_id, relaxator_params):
    status = get_current_status(pcdb, entry_id, relaxator_params)
    return status < relaxator_params['target_forces']


def get_current_status(pcdb, entry_id, relaxator_params, verbose=False):
    max_force = 1
    max_diag_stress = 1
    max_nondiag_stress = 1
    entry = pcdb.get_entry(entry_id)

    if entry is not None and 'properties' in entry and entry['properties'] is not None:
        if 'forces' in entry['properties'] and entry['properties']['forces'] is not None:
            forces = np.array(entry['properties']['forces']).reshape((-1, 3))
            max_force = np.max(np.apply_along_axis(np.linalg.norm, 1, forces))
        if 'stress' in entry['properties'] and entry['properties']['stress'] is not None:
            stress = np.array(entry['properties']['stress']).reshape((-1, 3))
            assert (stress.shape == (2, 3))
            max_diag_stress = np.abs(np.max(np.abs(stress[0])) - (relaxator_params['pressure'] / 1602.1766208))
            max_nondiag_stress = np.max(np.abs(stress[1]))
    else:
        pcm_log.debug('Bad entry')
        print(entry)
    if max(max_force, max_diag_stress, max_nondiag_stress) > relaxator_params['target_forces'] and verbose:
        if (max_force * max_diag_stress * max_nondiag_stress) == 1.0:
            print('No forces/stress information for entry: %s' % entry['_id'])
        else:
            print('Convergence status for entry: %s' % entry['_id'])
            print('Max Interatomic Force: %9.2E [eV/Ang]' % max_force)
            print('Max Diag stress      : %9.2E with pressure: %9.2E [eV/Ang^3]' % (max_diag_stress,
                                                                                    relaxator_params['pressure'] /
                                                                                    1602.1766208))
            print('Max NonDiag stress   : %9.2E [eV/Ang^3]' % max_nondiag_stress)
    return max(max_force, max_diag_stress, max_nondiag_stress)


if __name__ == '__main__':

    logging.basicConfig(level=logging.DEBUG)
    logger = logging.getLogger('pychemia')
    logger.addHandler(logging.NullHandler())
    logger.setLevel(logging.DEBUG)

    description = """Launch ab-initio codes for non-evaluated entries in a PyChemia Database"""

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('-t', '--host',
                        default='localhost', metavar='server', type=str,
                        help='Hostname or address (default: localhost)')
    parser.add_argument('-o', '--port',
                        default=27017, metavar='port', type=int,
                        help='MongoDB port (default: 27017)')
    parser.add_argument('-u', '--user',
                        default=None, metavar='username', type=str,
                        help='Username (default: None)')
    parser.add_argument('-p', '--passwd',
                        default=None, metavar='password', type=str,
                        help='Password (default: None)')
    parser.add_argument('-d', '--dbname',
                        default=None, metavar='dbname', type=str, nargs='+',
                        help='PyChemia Database name (default: None)')
    parser.add_argument('-b', '--binary',
                        default='vasp', metavar='path', type=str,
                        help='VASP binary (default: None)')
    parser.add_argument('-f', '--target_forces',
                        default=1E-3, metavar='x', type=float,
                        help='Target Forces (default: 1E-3)')
    parser.add_argument('--pressure_kB',
                        default=0.0, metavar='x', type=float,
                        help='Imposed Pressure (default: 0.0 kB)')
    parser.add_argument('-n', '--nparal',
                        default=1, metavar='N', type=int,
                        help='Number of parallel processes (default: 1)')
    parser.add_argument('-m', '--nmpiparal',
                        default=1, metavar='N', type=int,
                        help='Number of MPI parallel processes (default: 1)')
    parser.add_argument('-r', '--replicaset',
                        default=None, metavar='name', type=str,
                        help='ReplicaSet  (default: None)')
    parser.add_argument('-s', '--source_dir',
                        default=None, metavar='path', type=str,
                        help='Working Directory  (default: None)')
    parser.add_argument('-l', '--slater_path',
                        default=None, metavar='path', type=str,
                        help='Slater Path, used only for DFTB+ (default: None)')
    parser.add_argument('--evaluate_all', action='store_true',
                        help='Evaluate All  (default: No)')
    parser.add_argument('--waiting', action='store_true',
                        help='Waiting  (default: No)')
    parser.add_argument('--ssl', action='store_true',
                        help='Use SSL to connect to MongoDB  (default: No)')

    args = parser.parse_args()
    if args.dbname is None:
        parser.print_help()
        exit(1)

    print(args)
    db_settings = {'host': args.host, 'port': args.port, 'ssl': args.ssl, 'replicaset': args.replicaset}
    if args.user is not None:
        if args.passwd is None:
            raise ValueError('Password is mandatory if user is entered')
        db_settings['user'] = args.user
        db_settings['passwd'] = args.passwd

    relaxator_params = {'binary': args.binary,
                        'nmpiparal': args.nmpiparal,
                        'target_forces': args.target_forces,
                        'pressure': args.pressure_kB,
                        'source_dir': args.source_dir}

    print('pyChemia Evaluator using VASP')
    print('dbname    : %s' % args.dbname)
    print('source_dir: %s' % args.source_dir)
    print('host      : %s' % args.host)
    print('port      : %d' % args.port)
    print('user      : %s' % args.user)
    print('replicaset: %s' % args.replicaset)
    print('nparal    : %d' % args.nparal)
    print('nmpiparal : %d' % args.nmpiparal)
    print('binary    : %s' % str(args.binary))
    print('target-forces : %.2E' % args.target_forces)
    print('pressure_kB   : %.2E' % args.pressure_kB)
    print('evaluate_all  : %s' % str(args.evaluate_all))
    print('waiting       : %s' % str(args.waiting))
    print('ssl           : %s' % str(args.ssl))
    print('slater-path   : %s' % str(args.slater_path))

    print(db_settings)
    print(relaxator_params)

    worker = None
    if args.binary[:4].lower() == 'mais':
        worker = worker_maise
    elif args.binary[:4].lower() == 'dftb':
            worker = worker_dftb
    else:
        worker = worker_vasp

    evaluator = DirectEvaluator(db_settings=db_settings,
                                dbnames=args.dbname,
                                source_dir=args.source_dir,
                                is_evaluated=is_evaluated,
                                worker=worker,
                                worker_args=relaxator_params,
                                nconcurrent=args.nparal,
                                evaluate_all=args.evaluate_all)

    evaluator.run()



"""
PyChemia is a python library for automatize atomistic-level calculations. The library provide an API to manipulate
structures, store structures and properties in a Mongo database, search for new structures of a given composition,
interact with several atomistic simulation codes and visualize atomistic-related data

"""

from __future__ import print_function
import sys
import logging

__author__ = "Guillermo Avendano-Franco"
__copyright__ = "Copyright 2017"
__version__ = "0.17.3"
__email__ = "gufranco@mail.wvu.edu"
__status__ = "Development"
__date__ = "March 7, 2017"

try:
    import scipy

    HAS_SCIPY = True
except ImportError:
    HAS_SCIPY = False

try:
    try:
        import spglib as spg
    except ImportError:
        from pyspglib import spglib as spg
    HAS_SPGLIB = True
except ImportError:
    HAS_SPGLIB = False


try:
    import matplotlib
    HAS_MATPLOTLIB = True
    if 'matplotlib' not in sys.modules:
        matplotlib.use('agg')

except ImportError:
    HAS_MATPLOTLIB = False

try:
    import mayavi

    HAS_MAYAVI = True
except ImportError:
    HAS_MAYAVI = False

try:
    import pyhull

    HAS_PYHULL = True
except ImportError:
    HAS_PYHULL = False

try:
    import networkx

    HAS_NETWORKX = True
except ImportError:
    HAS_NETWORKX = False

try:
    import Scientific
    HAS_SCIENTIFIC = True
except ImportError:
    HAS_SCIENTIFIC = False

try:
    import pymongo

    if pymongo.version_tuple[0] < 3:
        HAS_PYMONGO = False
    else:
        HAS_PYMONGO = True
except ImportError:
    pymongo = None
    HAS_PYMONGO = False

try:
    import gridfs

    HAS_GRIDFS = True
except ImportError:
    gridfs = None
    HAS_GRIDFS = False


try:
    import ase
    HAS_ASE = True
except ImportError:
    HAS_ASE = False

try:
    import pymatgen
    HAS_PYMATGEN = True
except ImportError:
    HAS_PYMATGEN = False


pcm_log = logging.getLogger(__name__)
pcm_log.addHandler(logging.NullHandler())
from .core import Structure, Composition
from . import analysis
from . import db
from . import crystal
from . import io
from . import runner
from . import searcher
from . import utils
from . import web
from . import code
from . import population
from .core.from_file import structure_from_file
from pychemia.crystal import samples


def info():
    """
    Show basic information about PyChemia, its location and version.
        Also information about other libraries used by PyChemia
        both mandatory and optional

    """
    print('PyChemia\n--------\n')
    print('Version: ' + __version__)
    print('Path:    ' + __path__[0])
    print('Date:    ' + __date__)
    print()

    import sys
    print('Python version=' + sys.version + '\n')

    try:
        mm = __import__('pymongo')
        print('%10s %10s   %s' % ('pymongo', mm.version, mm.__path__[0]))
    except ImportError:
        print('pymongo Not Found')

    for modui in ['numpy', 'scipy', 'mayavi', 'Scientific', 'matplotlib',
                  'future', 'nose', 'coverage', 'spglib', 'pyhull', 'pymatgen', 'qmpy', ]:
        try:
            mm = __import__(modui)
            print('%10s %10s   %s' % (modui, mm.__version__, mm.__path__[0]))
        except ImportError:
            print('%10s %10s Not Found' % (modui, ''))

    try:
        import ase
        from ase import version as ase_version
        print('%10s %10s   %s' % ('ase', ase_version.version_base, ase.__path__[0]))
    except ImportError:
        print('%10s %10s Not Found' % ('ase', ''))


class Version:
    @staticmethod
    def full_version():
        return 'PyChemia Version=' + __version__ + ' from=' + __date__

    def __init__(self):
        pass
